#!/usr/bin/env python
__author__ = 'mahajrod'

import os
from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Tools.LinuxTools import CGAS
from RouToolPa.Routines import FileRoutines
from RouToolPa.Collections.General import SynDict



class BLASTPlus(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "blastn", path=path, max_threads=max_threads)

    def blast_search(self, blast_command, seqfile, database, outfile=None,
               blast_options=None, evalue=None, output_format=None):

        options = " -num_threads %i" % self.threads
        options += " -out %s" % outfile if outfile else ""
        options += " -db %s" % database
        options += " -query %s" % seqfile
        options += " %s" % blast_options if blast_options else ""
        options += " -evalue %s" % evalue if evalue else ""
        options += " -outfmt %i" % output_format if output_format else ""

        self.execute(options, cmd=blast_command)

    def parallel_blast(self, blast_command, seqfile, database, outfile=None,
                       blast_options=None, split_dir="splited_fasta",
                       splited_output_dir="splited_output_dir",
                       evalue=None, output_format=None,
                       threads=None, num_of_seqs_per_scan=None,
                       combine_output_to_single_file=True,
                       async_run=False,
                       external_process_pool=None):

        splited_dir = FileRoutines.check_path(split_dir)
        splited_out_dir = FileRoutines.check_path(splited_output_dir)
        self.safe_mkdir(splited_dir)
        self.safe_mkdir(splited_out_dir)

        number_of_files = num_of_seqs_per_scan if num_of_seqs_per_scan else 5 * threads if threads else 5 * self.threads
        self.split_fasta(seqfile, splited_dir, num_of_files=number_of_files)
        input_list_of_files = sorted(os.listdir(splited_dir))
        list_of_files = []

        for filename in input_list_of_files:
            filename_prefix = FileRoutines.split_filename(filename)[1]

            input_file = "%s%s" % (splited_dir, filename)
            output_file = "%s%s.hits" % (splited_out_dir, filename_prefix)

            list_of_files.append((input_file, output_file))

        options_list = []
        out_files = []

        for in_file, out_filename in list_of_files:

            options = " -out %s" % out_filename

            options += " -db %s" % database
            options += " -query %s" % in_file
            options += " %s" % blast_options if blast_options else ""
            options += " -evalue %s" % evalue if evalue else ""
            options += " -outfmt %i" % output_format if output_format else ""
            options_list.append(options)
            out_files.append(out_filename)

        self.parallel_execute(options_list, cmd=blast_command, threads=threads, async_run=async_run,
                              external_process_pool=external_process_pool)

        if combine_output_to_single_file:
            CGAS.cat(out_files, output=outfile)

    @staticmethod
    def extract_hits_from_tbl_output(blast_hits, output_file):
        hits = SynDict()
        hits.read(blast_hits, allow_repeats_of_key=True, key_index=0, value_index=1, separator="\t")
        hits.write(output_file, splited_values=True, separator="\t", values_separator=",")
        return hits


class BLASTn(Tool, BLASTPlus):
    """
    USAGE
      blastn [-h] [-help] [-import_search_strategy filename]
        [-export_search_strategy filename] [-task task_name] [-db database_name]
        [-dbsize num_letters] [-gilist filename] [-seqidlist filename]
        [-negative_gilist filename] [-entrez_query entrez_query]
        [-db_soft_mask filtering_algorithm] [-db_hard_mask filtering_algorithm]
        [-subject subject_input_file] [-subject_loc range] [-query input_file]
        [-out output_file] [-evalue evalue] [-word_size int_value]
        [-gapopen open_penalty] [-gapextend extend_penalty]
        [-perc_identity float_value] [-xdrop_ungap float_value]
        [-xdrop_gap float_value] [-xdrop_gap_final float_value]
        [-searchsp int_value] [-max_hsps_per_subject int_value] [-penalty penalty]
        [-reward reward] [-no_greedy] [-min_raw_gapped_score int_value]
        [-template_type type] [-template_length int_value] [-dust DUST_options]
        [-filtering_db filtering_database]
        [-window_masker_taxid window_masker_taxid]
        [-window_masker_db window_masker_db] [-soft_masking soft_masking]
        [-ungapped] [-culling_limit int_value] [-best_hit_overhang float_value]
        [-best_hit_score_edge float_value] [-window_size int_value]
        [-off_diagonal_range int_value] [-use_index boolean] [-index_name string]
        [-lcase_masking] [-query_loc range] [-strand strand] [-parse_deflines]
        [-outfmt format] [-show_gis] [-num_descriptions int_value]
        [-num_alignments int_value] [-html] [-max_target_seqs num_sequences]
        [-num_threads int_value] [-remote] [-version]

    DESCRIPTION
       Nucleotide-Nucleotide BLAST 2.2.28+

    OPTIONAL ARGUMENTS
     -h
       Print USAGE and DESCRIPTION;  ignore all other parameters
     -help
       Print USAGE, DESCRIPTION and ARGUMENTS; ignore all other parameters
     -version
       Print version number;  ignore other arguments

     *** Input query options
     -query <File_In>
       Input file name
       Default = `-'
     -query_loc <String>
       Location on the query sequence in 1-based offsets (Format: start-stop)
     -strand <String, `both', `minus', `plus'>
       Query strand(s) to search against database/subject
       Default = `both'

     *** General search options
     -task <String, Permissible values: 'blastn' 'blastn-short' 'dc-megablast'
                    'megablast' 'rmblastn' >
       Task to execute
       Default = `megablast'
     -db <String>
       BLAST database name
        * Incompatible with:  subject, subject_loc
     -out <File_Out>
       Output file name
       Default = `-'
     -evalue <Real>
       Expectation value (E) threshold for saving hits
       Default = `10'
     -word_size <Integer, >=4>
       Word size for wordfinder algorithm (length of best perfect match)
     -gapopen <Integer>
       Cost to open a gap
     -gapextend <Integer>
       Cost to extend a gap
     -penalty <Integer, <=0>
       Penalty for a nucleotide mismatch
     -reward <Integer, >=0>
       Reward for a nucleotide match
     -use_index <Boolean>
       Use MegaBLAST database index
     -index_name <String>
       MegaBLAST database index name

     *** BLAST-2-Sequences options
     -subject <File_In>
       Subject sequence(s) to search
        * Incompatible with:  db, gilist, seqidlist, negative_gilist,
       db_soft_mask, db_hard_mask
     -subject_loc <String>
       Location on the subject sequence in 1-based offsets (Format: start-stop)
        * Incompatible with:  db, gilist, seqidlist, negative_gilist,
       db_soft_mask, db_hard_mask, remote

     *** Formatting options
     -outfmt <String>
       alignment view options:
         0 = pairwise,
         1 = query-anchored showing identities,
         2 = query-anchored no identities,
         3 = flat query-anchored, show identities,
         4 = flat query-anchored, no identities,
         5 = XML Blast output,
         6 = tabular,
         7 = tabular with comment lines,
         8 = Text ASN.1,
         9 = Binary ASN.1,
        10 = Comma-separated values,
        11 = BLAST archive format (ASN.1)

       Options 6, 7, and 10 can be additionally configured to produce
       a custom format specified by space delimited format specifiers.
       The supported format specifiers are:
            qseqid means Query Seq-id
               qgi means Query GI
              qacc means Query accesion
           qaccver means Query accesion.version
              qlen means Query sequence length
            sseqid means Subject Seq-id
         sallseqid means All subject Seq-id(s), separated by a ';'
               sgi means Subject GI
            sallgi means All subject GIs
              sacc means Subject accession
           saccver means Subject accession.version
           sallacc means All subject accessions
              slen means Subject sequence length
            qstart means Start of alignment in query
              qend means End of alignment in query
            sstart means Start of alignment in subject
              send means End of alignment in subject
              qseq means Aligned part of query sequence
              sseq means Aligned part of subject sequence
            evalue means Expect value
          bitscore means Bit score
             score means Raw score
            length means Alignment length
            pident means Percentage of identical matches
            nident means Number of identical matches
          mismatch means Number of mismatches
          positive means Number of positive-scoring matches
           gapopen means Number of gap openings
              gaps means Total number of gaps
              ppos means Percentage of positive-scoring matches
            frames means Query and subject frames separated by a '/'
            qframe means Query frame
            sframe means Subject frame
              btop means Blast traceback operations (BTOP)
           staxids means Subject Taxonomy ID(s), separated by a ';'
         sscinames means Subject Scientific Name(s), separated by a ';'
         scomnames means Subject Common Name(s), separated by a ';'
        sblastnames means Subject Blast Name(s), separated by a ';'
                 (in alphabetical order)
        sskingdoms means Subject Super Kingdom(s), separated by a ';'
                 (in alphabetical order)
            stitle means Subject Title
        salltitles means All Subject Title(s), separated by a '<>'
           sstrand means Subject Strand
             qcovs means Query Coverage Per Subject
           qcovhsp means Query Coverage Per HSP
       When not provided, the default value is:
       'qseqid sseqid pident length mismatch gapopen qstart qend sstart send
       evalue bitscore', which is equivalent to the keyword 'std'
       Default = `0'
     -show_gis
       Show NCBI GIs in deflines?
     -num_descriptions <Integer, >=0>
       Number of database sequences to show one-line descriptions for
       Not applicable for outfmt > 4
       Default = `500'
        * Incompatible with:  max_target_seqs
     -num_alignments <Integer, >=0>
       Number of database sequences to show alignments for
       Default = `250'
        * Incompatible with:  max_target_seqs
     -html
       Produce HTML output?

     *** Query filtering options
     -dust <String>
       Filter query sequence with DUST (Format: 'yes', 'level window linker', or
       'no' to disable)
       Default = `20 64 1'
     -filtering_db <String>
       BLAST database containing filtering elements (i.e.: repeats)
     -window_masker_taxid <Integer>
       Enable WindowMasker filtering using a Taxonomic ID
     -window_masker_db <String>
       Enable WindowMasker filtering using this repeats database.
     -soft_masking <Boolean>
       Apply filtering locations as soft masks
       Default = `true'
     -lcase_masking
       Use lower case filtering in query and subject sequence(s)?

     *** Restrict search or results
     -gilist <String>
       Restrict search of database to list of GI's
        * Incompatible with:  negative_gilist, seqidlist, remote, subject,
       subject_loc
     -seqidlist <String>
       Restrict search of database to list of SeqId's
        * Incompatible with:  gilist, negative_gilist, remote, subject,
       subject_loc
     -negative_gilist <String>
       Restrict search of database to everything except the listed GIs
        * Incompatible with:  gilist, seqidlist, remote, subject, subject_loc
     -entrez_query <String>
       Restrict search with the given Entrez query
        * Requires:  remote
     -db_soft_mask <String>
       Filtering algorithm ID to apply to the BLAST database as soft masking
        * Incompatible with:  db_hard_mask, subject, subject_loc
     -db_hard_mask <String>
       Filtering algorithm ID to apply to the BLAST database as hard masking
        * Incompatible with:  db_soft_mask, subject, subject_loc
     -perc_identity <Real, 0..100>
       Percent identity
     -culling_limit <Integer, >=0>
       If the query range of a hit is enveloped by that of at least this many
       higher-scoring hits, delete the hit
        * Incompatible with:  best_hit_overhang, best_hit_score_edge
     -best_hit_overhang <Real, (>=0 and =<0.5)>
       Best Hit algorithm overhang value (recommended value: 0.1)
        * Incompatible with:  culling_limit
     -best_hit_score_edge <Real, (>=0 and =<0.5)>
       Best Hit algorithm score edge value (recommended value: 0.1)
        * Incompatible with:  culling_limit
     -max_target_seqs <Integer, >=1>
       Maximum number of aligned sequences to keep
       Not applicable for outfmt <= 4
       Default = `500'
        * Incompatible with:  num_descriptions, num_alignments

     *** Discontiguous MegaBLAST options
     -template_type <String, `coding', `coding_and_optimal', `optimal'>
       Discontiguous MegaBLAST template type
        * Requires:  template_length
     -template_length <Integer, Permissible values: '16' '18' '21' >
       Discontiguous MegaBLAST template length
        * Requires:  template_type

     *** Statistical options
     -dbsize <Int8>
       Effective length of the database
     -searchsp <Int8, >=0>
       Effective length of the search space
     -max_hsps_per_subject <Integer, >=0>
       Override maximum number of HSPs per subject to save for ungapped searches
       (0 means do not override)
       Default = `0'

     *** Search strategy options
     -import_search_strategy <File_In>
       Search strategy to use
        * Incompatible with:  export_search_strategy
     -export_search_strategy <File_Out>
       File name to record the search strategy used
        * Incompatible with:  import_search_strategy

     *** Extension options
     -xdrop_ungap <Real>
       X-dropoff value (in bits) for ungapped extensions
     -xdrop_gap <Real>
       X-dropoff value (in bits) for preliminary gapped extensions
     -xdrop_gap_final <Real>
       X-dropoff value (in bits) for final gapped alignment
     -no_greedy
       Use non-greedy dynamic programming extension
     -min_raw_gapped_score <Integer>
       Minimum raw gapped score to keep an alignment in the preliminary gapped and
       traceback stages
     -ungapped
       Perform ungapped alignment only?
     -window_size <Integer, >=0>
       Multiple hits window size, use 0 to specify 1-hit algorithm
     -off_diagonal_range <Integer, >=0>
       Number of off-diagonals to search for the 2nd hit, use 0 to turn off
       Default = `0'

     *** Miscellaneous options
     -parse_deflines
       Should the query and subject defline(s) be parsed?
     -num_threads <Integer, >=1>
       Number of threads (CPUs) to use in the BLAST search
       Default = `1'
        * Incompatible with:  remote
     -remote
       Execute search remotely?
        * Incompatible with:  gilist, seqidlist, negative_gilist, subject_loc,
       num_threads
    """
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "blastn", path=path, max_threads=max_threads)

    def search(self, seqfile, database, outfile=None,
               blast_options=None, evalue=None, output_format=None):

        self.blast_search("blastn", seqfile, database, outfile=outfile, blast_options=blast_options,
                          evalue=evalue, output_format=output_format)

    def parallel_blastn(self, seqfile, database, outfile=None,
                        blast_options=None, split_dir="splited_fasta",
                        splited_output_dir="splited_output_dir",
                        evalue=None, output_format=None,
                        threads=None, num_of_seqs_per_scan=None,
                        combine_output_to_single_file=True):

        self.parallel_blast("blastn", seqfile, database, outfile=outfile,
                            blast_options=blast_options, split_dir=split_dir,
                            splited_output_dir=splited_output_dir,
                            evalue=evalue, output_format=output_format,
                            threads=threads, num_of_seqs_per_scan=num_of_seqs_per_scan,
                            combine_output_to_single_file=combine_output_to_single_file)


class BLASTp(Tool, BLASTPlus):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "blastp", path=path, max_threads=max_threads)

    def search(self, seqfile, database, outfile=None,
               blast_options=None, evalue=None, output_format=None):

        self.blast_search("blastp", seqfile, database, outfile=outfile, blast_options=blast_options,
                          evalue=evalue, output_format=output_format)

    def parallel_blastp(self, seqfile, database, outfile=None,
                        blast_options=None, split_dir="splited_fasta",
                        splited_output_dir="splited_output_dir",
                        evalue=None, output_format=None,
                        threads=None, num_of_seqs_per_scan=None,
                        combine_output_to_single_file=True):

        self.parallel_blast("blastp", seqfile, database, outfile=outfile,
                            blast_options=blast_options, split_dir=split_dir,
                            splited_output_dir=splited_output_dir,
                            evalue=evalue, output_format=output_format,
                            threads=threads, num_of_seqs_per_scan=num_of_seqs_per_scan,
                            combine_output_to_single_file=combine_output_to_single_file)


class DustMasker(Tool):
    def __init__(self, path="", max_threads=4):
        """
        USAGE
          dustmasker [-h] [-help] [-xmlhelp] [-in input_file_name]
            [-out output_file_name] [-window window_size] [-level level]
            [-linker linker] [-infmt input_format] [-outfmt output_format]
            [-parse_seqids] [-version-full]

        DESCRIPTION
           Low complexity region masker based on Symmetric DUST algorithm

        OPTIONAL ARGUMENTS
         -h
           Print USAGE and DESCRIPTION;  ignore all other parameters
         -help
           Print USAGE, DESCRIPTION and ARGUMENTS; ignore all other parameters
         -xmlhelp
           Print USAGE, DESCRIPTION and ARGUMENTS in XML format; ignore all other
           parameters
         -in <File_In>
           input file name
           Default = `-'
         -out <File_Out>
           output file name
           Default = `-'
         -window <Integer>
           DUST window length
           Default = `64'
         -level <Integer>
           DUST level (score threshold for subwindows)
           Default = `20'
         -linker <Integer>
           DUST linker (how close masked intervals should be to get merged together).
           Default = `1'
         -infmt <String>
           input format (possible values: fasta, blastdb)
           Default = `fasta'
         -outfmt <String, `acclist', `fasta', `interval', `maskinfo_asn1_bin',
                          `maskinfo_asn1_text', `maskinfo_xml', `seqloc_asn1_bin',
                          `seqloc_asn1_text', `seqloc_xml'>
           output format
           Default = `interval'
         -parse_seqids
           Parse Seq-ids in FASTA input
         -version-full
           Print extended version data;  ignore other arguments
        """
        Tool.__init__(self, "dustmasker", path=path, max_threads=max_threads)

    def mask(self, input_file, output_file, output_format="maskinfo_asn1_bin", input_format="fasta", parse_seqids=True):
        options = " "
        options += " -in %s" % input_file
        options += " -infmt %s" % input_format
        options += " -outfmt %s" % output_format
        options += " -parse_seqids" if parse_seqids else ""
        options += " -out %s" % output_file

        self.execute(options)


class MakeBLASTDb(Tool):
    def __init__(self, path="", max_threads=4):
        """
        USAGE
          makeblastdb [-h] [-help] [-in input_file] [-input_type type]
            -dbtype molecule_type [-title database_title] [-parse_seqids]
            [-hash_index] [-mask_data mask_data_files] [-mask_id mask_algo_ids]
            [-mask_desc mask_algo_descriptions] [-gi_mask]
            [-gi_mask_name gi_based_mask_names] [-out database_name]
            [-max_file_sz number_of_bytes] [-logfile File_Name] [-taxid TaxID]
            [-taxid_map TaxIDMapFile] [-version]

        DESCRIPTION
           Application to create BLAST databases, version 2.2.30+

        REQUIRED ARGUMENTS
         -dbtype <String, `nucl', `prot'>
           Molecule type of target db

        OPTIONAL ARGUMENTS
         -h
           Print USAGE and DESCRIPTION;  ignore all other parameters
         -help
           Print USAGE, DESCRIPTION and ARGUMENTS; ignore all other parameters
         -version
           Print version number;  ignore other arguments

         *** Input options
         -in <File_In>
           Input file/database name
           Default = `-'
         -input_type <String, `asn1_bin', `asn1_txt', `blastdb', `fasta'>
           Type of the data specified in input_file
           Default = `fasta'

         *** Configuration options
         -title <String>
           Title for BLAST database
           Default = input file name provided to -in argument
         -parse_seqids
           Option to parse seqid for FASTA input if set, for all other input types
           seqids are parsed automatically
         -hash_index
           Create index of sequence hash values.

         *** Sequence masking options
         -mask_data <String>
           Comma-separated list of input files containing masking data as produced by
           NCBI masking applications (e.g. dustmasker, segmasker, windowmasker)
         -mask_id <String>
           Comma-separated list of strings to uniquely identify the masking algorithm
            * Requires:  mask_data
            * Incompatible with:  gi_mask
         -mask_desc <String>
           Comma-separated list of free form strings to describe the masking algorithm
           details
            * Requires:  mask_id
         -gi_mask
           Create GI indexed masking data.
            * Requires:  parse_seqids
            * Incompatible with:  mask_id
         -gi_mask_name <String>
           Comma-separated list of masking data output files.
            * Requires:  mask_data, gi_mask

         *** Output options
         -out <String>
           Name of BLAST database to be created
           Default = input file name provided to -in argumentRequired if multiple
           file(s)/database(s) are provided as input
         -max_file_sz <String>
           Maximum file size for BLAST database files
           Default = `1GB'
         -logfile <File_Out>
           File to which the program log should be redirected

         *** Taxonomy options
         -taxid <Integer, >=0>
           Taxonomy ID to assign to all sequences
            * Incompatible with:  taxid_map
         -taxid_map <File_In>
           Text file mapping sequence IDs to taxonomy IDs.
           Format:<SequenceId> <TaxonomyId><newline>
            * Requires:  parse_seqids
            * Incompatible with:  taxid

        """
        Tool.__init__(self, "makeblastdb", path=path, max_threads=max_threads)

        #makes BLAST database from fasta file

    def make_db(self, input_file, db_title, mask_data, db_type, output_file=None, input_format="fasta", parse_seqids=True):
        # mask_data can be either string or list of strings
        options = " -dbtype %s" % db_type
        options += " -in %s" % input_file
        options += " -input_type %s" % input_format
        options += " -parse_seqids" if parse_seqids else ""
        options += " -title %s" % db_title
        options += " -out %s" % output_file if None else " -out %s" % db_title
        options += " -mask_data %s" % (mask_data if isinstance(mask_data, str) else ",".join(mask_data)) \
            if mask_data is not None else ""

        self.execute(options)

    def make_protein_db(self, input_file, db_title, mask_data, output_file=None, input_format="fasta", parse_seqids=True):
        # mask_data can be either string or list of strings
        self.make_db(input_file, db_title, mask_data, "prot", output_file=output_file,
                     input_format=input_format, parse_seqids=parse_seqids)

    def make_nucleotide_db(self, input_file, db_title, mask_data, output_file=None, input_format="fasta", parse_seqids=True):
        # mask_data can be either string or list of strings
        self.make_db(input_file, db_title, mask_data, "nucl", output_file=output_file,
                     input_format=input_format, parse_seqids=parse_seqids)


class BLASTDbCmd(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "blastdbcmd", path=path, max_threads=max_threads)

    def db_info(self, db_name):
        options = " -info"
        options += " -db %s" % db_name

        self.execute(options)


class Windowmasker(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "windowmasker", path=path, max_threads=max_threads)

    def parse_options(self, mode, input_file, output_file, input_format="fasta", counts_format=None,
                      parse_seqids=False, counts_file=None, output_format=None):
        """
        description https://www.ncbi.nlm.nih.gov/IEB/ToolBox/CPP_DOC/lxr/source/src/app/winmasker/
        """
        if mode == "counts":
            options = " -mk_counts"
        elif mode == "convert":
            options = " -convert"
        elif mode == "mask" :
            if counts_file is None:
                raise ValueError("Counts file was not set")
            options = " -ustat %s " % counts_file
        else:
            raise ValueError("Wrong mode.")

        options += " -parse_seqids" if parse_seqids else ""
        options += " -in %s" % input_file
        options += " -out %s" % output_file
        options += " -infmt %s" % input_format if input_format else ""
        options += " -sformat %s " % counts_format if counts_format else ""
        options += " -outfmt %s" % output_format if output_format else ""

        return options

    def construct_counts_file(self, input_file, output_file, input_format="fasta", counts_format="obinary"):

        options = self.parse_options("counts", input_file, output_file, input_format=input_format,
                                     counts_format=counts_format,
                                     #parse_seqids=True
                                     )
        self.execute(options=options)

    def mask(self, input_file, counts_file, output_file, input_format="fasta", output_format="interval"):
        options = self.parse_options("mask", input_file, output_file, input_format=input_format,
                                     output_format=output_format,
                                     #parse_seqids=True,
                                     counts_file=counts_file)
        self.execute(options=options)

    @staticmethod
    def convert_interval_format_to_gff(input, output_gff, source="windowmasker", feature_type="gff"):
        with open(input, "r") as in_fd:
            with open(output_gff, "w") as out_fd:
                #line = in_fd.next()
                #seq_id = line.strip().split()[0][1:]
                for line in in_fd:
                    if line[0] == ">":
                        seq_id = line.strip().split()[0][1:]
                    else:
                        start, stop = map(lambda s: int(s) + 1, line.strip().split(" - ")) # convert coordinates to 1-based
                        gff_string = "%s\t%s\t%s\t%i\t%i\t.\t.\t.\t.\n" % (seq_id, source, feature_type, start, stop)
                        out_fd.write(gff_string)

    def masking(self, input_file, output_prefix, input_format="fasta", counts_format="obinary",
                masking_format="interval", source="windowmasker", feature_type="repeat"):

        counts_file = "%s.counts" % output_prefix
        masking_file = "%s.windowmasker" % output_prefix
        masking_gff_file = "%s.windowmasker.gff" % output_prefix

        self.construct_counts_file(input_file, counts_file, input_format=input_format, counts_format=counts_format)

        self.mask(input_file, counts_file, masking_file, input_format="fasta", output_format=masking_format)

        self.convert_interval_format_to_gff(masking_file, masking_gff_file, source=source, feature_type=feature_type)


if __name__ == "__main__":

    blast_plus = BLASTPlus()
    workdir = "/home/mahajrod/genetics/nxf/annotation/test/Dmel/nxf1/"
    os.chdir(workdir)
    blast_plus.make_blast_plus_db("/home/mahajrod/genetics/nxf/annotation/test/Dmel/nxf1/dmel-all-chromosome-r5.54.fasta",
                                  "dmel_all_chromosome.asnb", "dmel_all_chromosome")


#!/usr/bin/env python
from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Routines import FileRoutines


class GUIDANCE2(Tool):
    def __init__(self, path="", max_threads=1):
        Tool.__init__(self, "guidance.pl", path=path, max_threads=max_threads)

    def parse_common_options(self, seq_file=None, output_prefix=None, output_dir=None, msa_tool='prank',
                             seq_type=None, bootstrap_number=100, genetic_code=1, threads=None,
                             msa_tool_options=None, seq_cutoff=None, col_cutoff=None, mafft_bin=None,
                             prank_bin=None, muscle_bin=None, pagan_bin=None, ruby_bin=None, program=None):
        """
        1) Nuclear Standard
        15) Nuclear Blepharisma
        6) Nuclear Ciliate
        10) Nuclear Euplotid
        2) Mitochondria Vertebrate
        5) Mitochondria Invertebrate
        3) Mitochondria Yeast
        13) Mitochondria Ascidian
        9) Mitochondria Echinoderm
        14) Mitochondria Flatworm
        4) Mitochondria Protozoan
        """
        options = " --outDir %s" % output_dir if output_dir else ""
        options += " --program %s" % program if program else ""
        options += " --proc_num %i" % (threads if threads else self.threads)
        options += " --msaProgram %s" % msa_tool
        options += " --MSA_Param %s" % msa_tool_options if msa_tool_options else ""
        options += " --seqType %s" % seq_type if seq_type else ""
        options += " --bootstraps %i" % bootstrap_number if bootstrap_number else ""
        options += " --genCode %i" % genetic_code if genetic_code else ""
        options += " --seqCutoff %f" % seq_cutoff if seq_cutoff else ""
        options += " --colCutoff %f" % col_cutoff if col_cutoff else ""

        options += " --mafft %s" % mafft_bin if mafft_bin else ""
        options += " --prank %s" % prank_bin if prank_bin else ""
        options += " --muscle %s" % muscle_bin if muscle_bin else ""
        options += " --pagan %s" % pagan_bin if pagan_bin else ""
        options += " --ruby %s" % ruby_bin if ruby_bin else ""

        options += " --seqFile %s" % seq_file if seq_file else ""
        options += " --dataset %s" % output_prefix if output_prefix else ""

        return options

    def align(self, seq_file, output_prefix, output_dir, msa_tool='prank',
              seq_type=None, bootstrap_number=100, genetic_code=1, threads=None,
              msa_tool_options=None, seq_cutoff=None, col_cutoff=None, mafft_bin=None,
              prank_bin=None, muscle_bin=None, pagan_bin=None, ruby_bin=None, program=None):

        options = self.parse_common_options(seq_file=seq_file, output_prefix=output_prefix,
                                            output_dir=output_dir, msa_tool=msa_tool,
                                            seq_type=seq_type, bootstrap_number=bootstrap_number,
                                            genetic_code=genetic_code, threads=threads,
                                            msa_tool_options=msa_tool_options, seq_cutoff=seq_cutoff,
                                            col_cutoff=col_cutoff, mafft_bin=mafft_bin,
                                            prank_bin=prank_bin, muscle_bin=muscle_bin,
                                            pagan_bin=pagan_bin, ruby_bin=ruby_bin, program=program)
        self.execute(options)

    def parallel_align(self, list_of_files, output_dir, msa_tool='prank',
                       seq_type=None, bootstrap_number=100, genetic_code=1, threads=None,
                       msa_tool_options=None, seq_cutoff=None, col_cutoff=None, mafft_bin=None,
                       prank_bin=None, muscle_bin=None, pagan_bin=None, ruby_bin=None, program=None,
                       cmd_log_file=None,
                       cpus_per_task=1,
                       handling_mode="local",
                       job_name=None,
                       log_prefix=None,
                       error_log_prefix=None,
                       max_jobs=None,
                       max_running_time=None,
                       max_memory_per_node=None,
                       max_memmory_per_cpu=None,
                       modules_list=None,
                       environment_variables_dict=None):

        common_options = self.parse_common_options(output_dir=output_dir, msa_tool=msa_tool,
                                                   seq_type=seq_type, bootstrap_number=bootstrap_number,
                                                   genetic_code=genetic_code, threads=threads,
                                                   msa_tool_options=msa_tool_options, seq_cutoff=seq_cutoff,
                                                   col_cutoff=col_cutoff, mafft_bin=mafft_bin,
                                                   prank_bin=prank_bin, muscle_bin=muscle_bin,
                                                   pagan_bin=pagan_bin, ruby_bin=ruby_bin, program=program)

        FileRoutines.safe_mkdir(output_dir)
        options_list = []
        for filename in list_of_files:
            basename = FileRoutines.split_filename(filename)[1]
            op = common_options
            op += " --seqFile %s" % filename
            op += " --dataset %s" % basename
            options_list.append(op)
        if handling_mode == "local":
            self.parallel_execute(options_list)
        elif handling_mode == "slurm":

            cmd_list = ["%s%s %s" % ((self.path + "/") if self.path else "", self.cmd, options) for options in options_list]
            self.slurm_run_multiple_jobs_in_wrap_mode(cmd_list,
                                                      cmd_log_file,
                                                      max_jobs=max_jobs,
                                                      job_name=job_name,
                                                      log_prefix=log_prefix,
                                                      error_log_prefix=error_log_prefix,
                                                      cpus_per_node=None,
                                                      max_running_jobs=None,
                                                      max_running_time=max_running_time,
                                                      cpus_per_task=cpus_per_task,
                                                      max_memory_per_node=max_memory_per_node,
                                                      max_memmory_per_cpu=max_memmory_per_cpu,
                                                      modules_list=modules_list,
                                                      environment_variables_dict=environment_variables_dict)

    def parallel_codon_alignment(self, list_of_files, output_dir, msa_tool='prank',
                                 bootstrap_number=100, genetic_code=1, threads=None,
                                 msa_tool_options=None, seq_cutoff=None, col_cutoff=None, mafft_bin=None,
                                 prank_bin=None, muscle_bin=None, pagan_bin=None, ruby_bin=None, program=None,
                                 cmd_log_file=None,
                                 cpus_per_task=1,
                                 handling_mode="local",
                                 job_name=None,
                                 log_prefix=None,
                                 error_log_prefix=None,
                                 max_jobs=None,
                                 max_running_time=None,
                                 max_memory_per_node=None,
                                 max_memmory_per_cpu=None,
                                 modules_list=None,
                                 environment_variables_dict=None):

        self.parallel_align(list_of_files, output_dir, msa_tool=msa_tool,
                            seq_type="codon", bootstrap_number=bootstrap_number,
                            genetic_code=genetic_code, threads=threads,
                            msa_tool_options=msa_tool_options, seq_cutoff=seq_cutoff, col_cutoff=col_cutoff,
                            mafft_bin=mafft_bin,
                            prank_bin=prank_bin, muscle_bin=muscle_bin,
                            pagan_bin=pagan_bin, ruby_bin=ruby_bin, program=program,
                            cmd_log_file=cmd_log_file,
                            cpus_per_task=cpus_per_task,
                            handling_mode=handling_mode,
                            job_name=job_name,
                            log_prefix=log_prefix,
                            error_log_prefix=error_log_prefix,
                            max_jobs=max_jobs,
                            max_running_time=max_running_time,
                            max_memory_per_node=max_memory_per_node,
                            max_memmory_per_cpu=max_memmory_per_cpu,
                            modules_list=modules_list,
                            environment_variables_dict=environment_variables_dict)



#!/usr/bin/env python

from RouToolPa.Tools.Abstract import Tool


class CCS(Tool):

    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "ccs", path=path, max_threads=max_threads)

    @staticmethod
    def parse_common_options(max_subread_len=None, min_subread_length=None, min_subread_num_per_circle=None):

        options = ""
        options += " --maxLength %i" % max_subread_len if max_subread_len else ""
        options += " --minLength %i" % min_subread_length if min_subread_length else ""
        options += " --minPasses %i" % min_subread_num_per_circle if min_subread_num_per_circle else ""

        return options

    def get_consensus(self, input_file, output_file, max_subread_len=None, min_subread_length=None,
                      min_subread_num_per_circle=None):

        options = self.parse_common_options(max_subread_len=max_subread_len,
                                            min_subread_length=min_subread_length,
                                            min_subread_num_per_circle=min_subread_num_per_circle)
        options += " %s" % input_file
        options += " %s" % output_file

        self.execute(options=options)



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'
import os
import shutil
import numpy as np
from scipy.signal import argrelextrema
import matplotlib


matplotlib.use('Agg')
os.environ['MPLCONFIGDIR'] = '/tmp/'
import matplotlib.pyplot as plt
plt.ioff()
from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Routines import MatplotlibRoutines, MathRoutines


class Jellyfish(Tool):
    """
    Several subcommands are not implemented: qhisto, qdump, qmerge, cite
    Not all options were implemented for count subcommand

    """
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "jellyfish", path=path, max_threads=max_threads)

    def count(self, in_file, out_file, kmer_length=23, hash_size=1000000, count_both_strands=False,
              lower_count=None, upper_count=None, generators=None):
        # IMPORTANT! Not all options were implemented
        if (lower_count is not None) and (upper_count is not None):
            if lower_count > upper_count:
                raise ValueError("Upper limit for kmer counts is less than lower")

        options = "-m %i" % kmer_length
        options += " -s %s" % str(hash_size)
        options += " -t %i" % self.threads
        options += " -o %s" % out_file
        options += " -C" if count_both_strands else ""
        options += " -L %i" % lower_count if lower_count is not None else ""
        options += " -U %i" % upper_count if upper_count is not None else ""

        input_files = [in_file] if isinstance(in_file, str) else in_file
        filetypes_list = []
        for filename in input_files:
            filetype = self.detect_filetype_by_extension(filename)
            if filetype not in filetypes_list:
                filetypes_list.append(filetype)

        if len(filetypes_list) > 1:
            raise ValueError("Mix of filetypes in input files: %s" % ",".join(filetypes_list))

        if generators:
            options += " -g %s" % generators
            options += " -G %i" % self.threads

        else:

            if filetypes_list[0] == "fasta" or filetypes_list[0] == "fastq" or filetypes_list[0] is None:
                if filetypes_list[0] is None:
                    print("Warning!!! Type of input files was not recognized. Treating them as fasta...")
                cmd = "jellyfish count"
                options += " %s" % " ".join(input_files)
            elif filetypes_list[0] == "bzip":
                file_options = "bzcat %s" % " ".join(input_files)
                cmd = "%s | jellyfish count" % file_options
                options += " /dev/fd/0"
            elif filetypes_list[0] == "gz":
                file_options = "zcat %s" % " ".join(input_files)
                cmd = "%s | jellyfish count" % file_options
                options += " /dev/fd/0"
            elif filetypes_list[0] == "bam" or filetypes_list[0] == "sam" or filetypes_list[0] == "cram":
                cmd = "jellyfish count"
                if len(input_files) > 1:
                    raise ValueError("Jellyfish can deal with one sam/bam/cram file at a time")
                options += " --sam %s" % in_file[0]

        self.execute(options, cmd=cmd)

    def stats(self, in_file, out_file, lower_count=None, upper_count=None):

        if (lower_count is not None) and (upper_count is not None):
            if lower_count > upper_count:
                raise ValueError("Upper limit for kmer counts is less than lower")

        options = " -o %s" % out_file
        options += " -L %i" % lower_count if lower_count is not None else ""
        options += " -U %i" % upper_count if upper_count is not None else ""
        options += " %s" % in_file

        self.execute(options, cmd="jellyfish stats")

    def histo(self, in_file, out_file, bin_width=1, lower_count=1, upper_count=100000000,
              include_absent_kmers=False):

        if (lower_count is not None) and (upper_count is not None):
            if lower_count > upper_count:
                raise ValueError("Upper limit for kmer counts is less than lower")

        options = " -o %s" % out_file
        options += " -t %i" % self.threads
        options += " -l %i" % lower_count
        options += " -h %i" % upper_count
        options += " -f" if include_absent_kmers else ""
        options += " -i %i" % bin_width
        options += " %s" % in_file

        self.execute(options, cmd="jellyfish histo")

    def dump(self, in_file, out_file, lower_count=None, upper_count=None,
             column_format=True, tab_separator=True):

        if (lower_count is not None) and (upper_count is not None):
            if lower_count > upper_count:
                raise ValueError("Upper limit for kmer counts is less than lower")

        options = " -o %s" % out_file
        options += " -L %i" % lower_count if lower_count is not None else ""
        options += " -U %i" % upper_count if upper_count is not None else ""
        options += " -c" if column_format else ""
        options += " -t" if tab_separator else ""
        options += " %s" % in_file

        self.execute(options, cmd="jellyfish dump")

    def dump_kmers(self, in_file, output_prefix, lower_count=None, upper_count=None):

        if (lower_count is not None) and (upper_count is not None):
            if lower_count > upper_count:
                raise ValueError("Upper limit for kmer counts is less than lower")

        options = " -t"     # use tab separator
        options += " -L %i" % lower_count if lower_count is not None else ""
        options += " -U %i" % upper_count if upper_count is not None else ""
        options += " -c"    # column_format
        options += " %s" % in_file
        options += " | tee %s.counts | cut -f 1 " % output_prefix
        options += " > %s.kmers" % output_prefix

        self.execute(options, cmd="jellyfish dump")

    def query(self, sequence_file, jf_database, output_file):

        options = " -s %s" % sequence_file
        options += " -o %s" % output_file
        options += " %s" % jf_database

        self.execute(options, cmd="jellyfish query")

    def parallel_query(self, sequence_file, jf_database, output_file, splited_input_dir="splited_fasta",
                       per_sequence_analysis=False, parsing_mode="parse", splited_output_dir="splited_output",
                       retain_intermediate_file=False):

        self.split_fasta(sequence_file, splited_input_dir, num_of_recs_per_file=1 if per_sequence_analysis else None,
                         num_of_files=None if per_sequence_analysis else 5 * self.threads,
                         output_prefix="splited_fasta", parsing_mode=parsing_mode)
        options_list = []

        for filename in os.listdir(splited_input_dir):
            input_file = "%s/%s" % (splited_input_dir, filename)
            output_file = "%s/%s.kmer.counts" % (splited_output_dir, filename)

            options = " -s %s" % input_file
            options += " -o %s" % output_file
            options += " %s" % jf_database

            options_list.append(options)

        self.parallel_execute(options_list, cmd="jellyfish query")

        os.system("cat %s/* > %s" %(splited_output_dir, output_file))

        if not retain_intermediate_file:
            shutil.rmtree(splited_input_dir)
            shutil.rmtree(splited_output_dir)

    def get_kmer_list(self, in_file, out_prefix, kmer_length=23, hash_size=1000000, count_both_strands=False,
                      lower_count=None, upper_count=None):
        base_file = "%s.%i.jf" % (out_prefix, kmer_length)
        self.count(in_file, base_file, kmer_length=kmer_length, hash_size=hash_size,
                   count_both_strands=count_both_strands,
                   lower_count=lower_count, upper_count=upper_count)
        self.dump_kmers(in_file, out_prefix, lower_count=lower_count, upper_count=upper_count)



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from RouToolPa.Tools.Abstract import Tool


class BionanoSolve(Tool):
    """
    Requirements:
    1. python 2.7.5
    2. perl 5.14.X or 5.16.X
    3. R 3.1.2 or greater with R libraries data.table, igraph, intervals, MASS, parallel, XML
    (which requires a Linux system library libxml2), argparser
    4. glibc and gcc libraries (for older CPUs without AVX)
    5. minimum RAM size is 256 GB on at least 1 node, 32 GB on all nodes
    Running the assembly pipeline on a cluster requires a batch system that supports the Distributed
    Resource Management Application API (DRMAA). The submission hosts for the cluster also require
    the Python DRMAA module.
    """
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "", path=path, max_threads=max_threads)

    def two_enzyme_scaffolding(self, parameter_file, enzyme1_map, enzyme2_map,
                               enzyme1_name, enzyme2_name, assembly_fasta,
                               ref_aligner_path, output_prefix, output_dir="./",
                               enzyme1_manual_cut=None, enzyme2_manual_cut=None):
        options = " "
        options += " -b1 %s" % enzyme1_map
        options += " -b2 %s" % enzyme2_map
        options += " -e1 %s" % enzyme1_name
        options += " -e2 %s" % enzyme2_name
        options += " -m1 %s" % enzyme1_manual_cut
        options += " -m2 %s" % enzyme2_manual_cut

        options += " -N %s" % assembly_fasta
        options += " -R %s" % ref_aligner_path
        options += " -O %s" % output_dir
        options += " -t %s/%s.tar" % (output_dir, output_prefix)
        options += " -s %s/%s.status" % (output_dir, output_prefix)

        options += " %s" % parameter_file

        self.execute(options, cmd="runTGH.R", intepreter="Rscript")

    def error_correction(self):
        pass



__author__ = 'mahajrod'

from RouToolPa.Tools.Expression.Subread import Subread
from RouToolPa.Tools.Expression.Cufflinks import *
from RouToolPa.Tools.Expression.HTSeq import HTSeq


HTSeq = HTSeq()
Subread = Subread()
Gffread = Gffread()
Gtf2sam = Gtf2sam()
Cuffnorm = Cuffnorm()
Cuffdiff = Cuffdiff()
Cuffmerge = Cuffmerge()
Cufflinks = Cufflinks()
Cuffquant = Cuffquant()
Cuffcompare = Cuffcompare()



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

from RouToolPa.Tools.Abstract import JavaTool


class AddOrReplaceReadGroups(JavaTool):

    def __init__(self, java_path="", max_threads=4, jar_path="", max_memory="1g"):

        #JavaTool.__init__(self, "AddOrReplaceReadGroups.jar", java_path=java_path, max_threads=max_threads,
        #                  jar_path=jar_path, max_memory=max_memory)
        JavaTool.__init__(self, "picard.jar AddOrReplaceReadGroups", java_path=java_path, max_threads=max_threads,
                          jar_path=jar_path, max_memory=max_memory)

    def add_read_groups(self, input_bam, output_bam, RGID, RGLB, RGPL, RGSM, RGPU):

        options = " I= %s" % input_bam
        options += " O= %s" % output_bam
        options += " SORT_ORDER=coordinate"
        options += " RGID=%s" % RGID
        options += " RGLB=%s" % RGLB
        options += " RGPL=%s" % RGPL
        options += " RGSM=%s" % RGSM
        options += " RGPU=%s" % RGPU
        options += " CREATE_INDEX=True"

        self.execute(options)

if __name__ == "__main__":
    pass


#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

from RouToolPa.Tools.Abstract import Tool


class BioConvert(Tool):
    def __init__(self, path="", max_threads=1):
        Tool.__init__(self, "bfconvert", path=path, max_threads=max_threads)

    def convert(self, input_file, output_file):

        options = " %i" % input_file
        options += " %s" % output_file

        self.execute(options)

    def parallel_convert(self, list_of_files, output_directory, output_format=".tiff"):

        self.safe_mkdir(output_directory)
        options_list = []

        for filename in list_of_files:
            option = " %s" % filename
            option += " %s%s%s" % (self.check_path(output_directory), self.split_filename(filename)[1], output_format)
            options_list.append(option)

        self.parallel_execute(options_list)

if __name__ == "__main__":
    pass



__author__ = 'mahajrod'
import re
from RouToolPa.Parsers.Abstract import Record, Header, Metadata, Collection


class RecordFPKMTracing(Record):
    def __init__(self, chrom, pos, length, tracking_id, class_code, nearest_ref_id, gene_id, gene_short_name, tss_id,
                 coverage, FPKM, FPKM_conf_lo, FPKM_conf_hi, FPKM_status, description=None, flags=None):

        Record.__init__(self, chrom, pos,  description=description, flags=flags)
        # note that in .fpkm_tracking file locus coordinates are in python notation - starts from 0 and end nt is not included
        # probably, it is an error - check for every cufflinks version independently
        self.chrom = chrom                              # str
        self.length = length                            # int
        self.tracking_id = tracking_id                  # str
        self.class_code = class_code                    # str
        self.nearest_ref_id = nearest_ref_id            # str
        self.gene_id = gene_id                          # str
        self.gene_short_name = gene_short_name          # str
        self.tss_id = tss_id                            # str
        self.coverage = coverage                        # float or "-"
        self.FPKM = FPKM                                # float
        self.FPKM_conf_lo = FPKM_conf_lo                # float
        self.FPKM_conf_hi = FPKM_conf_hi                # float
        self.FPKM_status = FPKM_status                  # str

    def __str__(self):
        #tracking_id	class_code	nearest_ref_id	gene_id	gene_short_name	tss_id	locus	length	coverage	FPKM	FPKM_conf_lo	FPKM_conf_hi	FPKM_status
        return "\t".join([str(x) for x in (self.tracking_id, self.class_code, self.nearest_ref_id, self.gene_id,
                                           self.gene_short_name, self.tss_id, "%s:%i-%i" % (self.chrom, self.pos,
                                                                                            self.pos + self.length - 1),
                                           self.length, self.coverage, self.FPKM, self.FPKM_conf_lo, self.FPKM_conf_hi,
                                           self.FPKM_status)])


class HeaderFPKMTracking(Header):

    def __init__(self, header_string=None):
        if header_string is None:
            self.header = "tracking_id\tclass_code\tnearest_ref_id\tgene_id\tgene_short_name\ttss_id\tlocus\tlength\tcoverage\tFPKM\tFPKM_conf_lo\tFPKM_conf_hi\tFPKM_status"
        else:
            self.header = header_string

    def __str__(self):
        return self.header


class CollectionFPKMTracking(Collection):
    def __init__(self, metadata=None, record_list=None, header=None, input_file=None, from_file=False):
        Collection.__init__(self, metadata=metadata, record_list=record_list,
                            header=header if header else HeaderFPKMTracking(),
                            input_file=input_file, from_file=from_file)

    def read(self, input_file, input_in_python_coord_notation=True):
        self.metadata = None
        self.records = []
        with open(input_file, "r") as in_fd:
            self.header = in_fd.readline().strip()
            for line in in_fd:
                line_list = re.split("\t", line.strip())# line.strip().split("\t")
                if line[0] == "\t":
                    line_list.insert(0, "")

                chrom, pos = line_list[6].split(":")
                pos, end = [int(x) for x in pos.split("-")]

                if input_in_python_coord_notation:
                    pos += 1

                length = end - pos + 1
                cov_fpkm = [x if x == "-" else float(x) for x in line_list[8:-1]]
                packed_args = [chrom, pos, length] + line_list[:6] + cov_fpkm + [line_list[-1]]
                self.records.append(RecordFPKMTracing(*packed_args))

    def filter_by_expression(self, expression):
        filtered_records, filtered_out_records = self.filter_records_by_expression(expression)
        return CollectionFPKMTracking(metadata=self.metadata, header=self.header, record_list=filtered_records,
                                      from_file=False), \
               CollectionFPKMTracking(metadata=self.metadata, header=self.header, record_list=filtered_out_records,
                                      from_file=False)

if __name__ == "__main__":
    collection = CollectionFPKMTracking(from_file=True, input_file="/media/mahajrod/d9e6e5ee-1bf7-4dba-934e-3f898d9611c8/Data/LAN2xx/expression/Yeast_RNA_Seq/S1_Nagal_clout/genes.fpkm_tracking")
    collection.write("/media/mahajrod/d9e6e5ee-1bf7-4dba-934e-3f898d9611c8/Data/LAN2xx/expression/Yeast_RNA_Seq/S1_Nagal_clout/check.fpkm_tracking")


#!/usr/bin/env python

__author__ = 'mahajrod'

from RouToolPa.Tools.Abstract import JavaTool


class PrintReads(JavaTool):
    def __init__(self,  java_path="", max_threads=4, jar_path="", max_memory=None, timelog=None):
        JavaTool.__init__(self, "GenomeAnalysisTK.jar -T PrintReads", java_path=java_path,
                          max_threads=max_threads, jar_path=jar_path, max_memory=max_memory,
                          timelog=timelog)

    def get_recalled_reads(self, reference, input_bam, recal_table, output_bam):
        options = " -nct %i" % self.threads
        options += " -R %s" % reference
        options += " -I %s" % input_bam
        options += " -BQSR %s" % recal_table
        options += " -o %s" % output_bam

        self.execute(options)



#!/usr/bin/env python
import os
from RouToolPa.Tools.Abstract import Tool



class ApplyRecalibration():
    # http://www.broadinstitute.org/gatk/gatkdocs/org_broadinstitute_gatk_tools_walkers_variantrecalibration_ApplyRecalibration.html

    def apply_model(self, reference_file, input_vcf, tranchesfile, recalfile,
                    outputfile, mode="SNP", ts_filter_level=99.0, GATK_dir=""):
        # possible mode:
        # SNP
        # INDEL
        os.system("java -Xmx3g -jar %sGenomeAnalysisTK.jar -T ApplyRecalibration -R %s -input %s \
                   --ts_filter_level %f -tranchesFile %s -recalFile %s -mode %s -o %s"
                  % (GATK_dir, reference_file, input_vcf, ts_filter_level, tranchesfile, recalfile, mode, outputfile))



#!/usr/bin/env python
import os
from RouToolPa.Tools.Abstract import JavaTool



class ValidateVariants(JavaTool):
    # http://www.broadinstitute.org/gatk/gatkdocs/org_broadinstitute_sting_gatk_walkers_filters_VariantFiltration.html
    # default filters for indel and snp filtration were taken from GATK BestPractice
    def __init__(self,  java_path="", max_threads=4, jar_path="", max_memory=None, timelog=None):
        JavaTool.__init__(self, "GenomeAnalysisTK.jar -T ValidateVariants", java_path=java_path,
                          max_threads=max_threads, jar_path=jar_path, max_memory=max_memory,
                          timelog=timelog)

    @staticmethod
    def parse_common_options(reference, input_vcf, exclude_list=[], dbsnp=None, input_is_gvcf=False):
        options = " -R %s" % reference
        options += " -V %s" % input_vcf
        options += " --dbsnp %s" % dbsnp if dbsnp else ""
        for entry in exclude_list:
            options += " --validationTypeToExclude %s" % entry

        options += " --validateGVCF" if input_is_gvcf else ""

        return options

    def test_vcf_format(self, reference, input_vcf, input_is_gvcf=False):

        options = self.parse_common_options(reference, input_vcf, exclude_list=["ALL", ], input_is_gvcf=input_is_gvcf)

        self.execute(options=options)

    def index_vcf(self, reference, input_vcf, input_is_gvcf=False):
        self.test_vcf_format(reference, input_vcf, input_is_gvcf=input_is_gvcf)




#!/usr/bin/env python
from RouToolPa.Tools.Abstract import Tool


class MaskFasta(Tool):
    """
    Tool:    bedtools getfasta (aka fastaFromBed)
    Version: v2.17.0
    Summary: Extract DNA sequences into a fasta file based on feature coordinates.

    Usage:   bedtools getfasta [OPTIONS] -fi <fasta> -bed <bed/gff/vcf> -fo <fasta>

    Options:
        -fi	Input FASTA file
        -bed	BED/GFF/VCF file of ranges to extract from -fi
        -fo	Output file (can be FASTA or TAB-delimited)
        -name	Use the name field for the FASTA header
        -split	given BED12 fmt., extract and concatenate the sequencesfrom the BED "blocks" (e.g., exons)
        -tab	Write output in TAB delimited format.
            - Default is FASTA format.

        -s	Force strandedness. If the feature occupies the antisense,
            strand, the sequence will be reverse complemented.
            - By default, strand information is ignored.

    """

    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "bedtools maskfasta", path=path, max_threads=max_threads)

    def mask(self, input_fasta, output_fasta, masking_gff, softmasking=False, masking_character=None):

        options = " -bed %s" % masking_gff
        options += " -fi %s" % input_fasta
        options += " -fo %s" % output_fasta
        options += " -soft" if softmasking else ""
        options += (" -mc %s" % masking_character) if masking_character else ""

        self.execute(options)



__author__ = 'mahajrod'
import os
from collections import OrderedDict
import numpy as np
import pandas as pd
from Bio import SeqIO, AlignIO
from Bio.Seq import Seq
from Bio.Data import CodonTable
from Bio.Align import MultipleSeqAlignment
from Bio.SeqRecord import SeqRecord
from RouToolPa.Collections.General import SynDict, TwoLvlDict
from RouToolPa.Routines.Sequence import SequenceRoutines


class MultipleAlignmentRoutines(SequenceRoutines):
    def __init__(self):
        SequenceRoutines.__init__(self)
        self.tmp_count_dict = {"A": 0, "T": 0, "G": 0, "C": 0, "N": 0}
        self.strict_nucleotide_list = ["A", "T", "G", "C"]
        self.strict_nucleotide_set = {"A", "T", "G", "C", "N"}
        self.nucleotide_list = ["A", "T", "G", "C", "N"]
        self.nucleotide_set = {"A", "T", "G", "C", "N"}

    @staticmethod
    def get_general_statistics(alignment, verbose=False):
        number_of_sequences = len(alignment)
        length_of_alignment = alignment.get_alignment_length()
        if verbose:
            print ("Alignment statistics:")
            print ("\tNumber of sequences %i" % number_of_sequences)
            print ("\tLength %i" % length_of_alignment)
        return number_of_sequences, length_of_alignment

    @staticmethod
    def get_position_presence_matrix(alignment, gap_symbol="-", verbose=True):

        number_of_sequences = len(alignment)
        alignment_length = len(alignment[0])
        # converting alignment to numpy letter array stored by columns!
        alignment_array = np.array([list(rec) for rec in alignment], np.character, order="F")

        if verbose:
            print("%i sequences in alignment" % number_of_sequences)
            print("%i columns in alignment" % alignment_length)

        position_presence_array = np.array([[0 for letter in rec.seq] for rec in alignment], int, order="F")

        for column in range(0, alignment_length):
            column_list = alignment_array[:, column]
            number_of_gaps = 0
            for element in column_list:
                if element == gap_symbol:
                    number_of_gaps += 1
            for row in range(0, number_of_sequences):
                position_presence_array[row, column] = -number_of_gaps if alignment_array[row, column] == gap_symbol else number_of_sequences - number_of_gaps

        #for row in range(0, number_of_sequences):
        #    for column in range(0, alignment_length):
        #print alignment_array[0, ]
        #print alignment_array[:, 1]
        #print alignment_array
        #print position_presence_array
        return position_presence_array

    @staticmethod
    def count_nucleotides_per_position(alignment, verbose=True, gap_symbol="-", add_max_fraction=False,
                                       output_prefix=None,
                                       fraction_threshold=None, collapse_positions=False, ):
        number_of_sequences = len(alignment)
        alignment_length = len(alignment[0])
        # converting alignment to numpy letter array stored by columns!
        alignment_array = np.array([list(rec) for rec in alignment], np.character, order="F")

        if verbose:
            print("%i sequences in alignment" % number_of_sequences)
            print("%i columns in alignment" % alignment_length)

        nucleotide_list = ["A", "C", "G", "T", "R", "Y", "S", "W", "K", "M", "B", "D", "H", "V", "N", gap_symbol]
        count_df = pd.DataFrame(0, index=[i for i in range(0, alignment_length)], columns=nucleotide_list)

        for column in range(0, alignment_length):
            column_array = alignment_array[:, column]
            unique, counts = np.unique(column_array, return_counts=True)
            for index in range(0, len(unique)):
                # print type(unique[index])
                # print type(counts[index])
                # print str(unique[index])
                # print unique
                count_df[str(unique[index])][column] = counts[index]
        if output_prefix:
            count_df.to_csv("%s.only_counts.counts" % output_prefix,
                            sep="\t", index=True, index_label="pos")
        if add_max_fraction:

            count_df["max"] = count_df.apply(max, axis=1)
            count_df["max"] /= number_of_sequences
            count_df["max_letter"] = count_df.idxmax(axis=1)

            if output_prefix:
                count_df.to_csv("%s.with_max.counts" % output_prefix,
                                sep="\t", index=True, index_label="pos")

            if fraction_threshold:
                count_df = count_df[(count_df["max"] >= fraction_threshold) & (count_df["max_letter"] != gap_symbol)]

                if output_prefix:
                    count_df.to_csv("%s.with_max.min%.3f.counts" % (output_prefix, fraction_threshold),
                                    sep="\t", index=True, index_label="pos")
            if collapse_positions:
                prev_start = None
                prev_end = None
                collapsed_df = []
                for pos in count_df.index:
                    if prev_start is None:
                        prev_start = pos
                        prev_end = pos

                        continue
                    elif pos != prev_end + 1:
                        # python interval notation!

                        collapsed_df.append([prev_start, prev_end + 1])
                        prev_start = pos
                        prev_end = pos
                    else:
                        prev_end += 1

                collapsed_df.append([prev_start, prev_end + 1])

                collapsed_df = pd.DataFrame(collapsed_df, columns=["start", "end"])
                collapsed_df["length"] = collapsed_df["end"] - collapsed_df["start"]
                if output_prefix:
                    collapsed_df.to_csv("%s.with_max.min%.3f.regions" % (output_prefix, fraction_threshold),
                                        sep="\t", index=False)

        return count_df

    def get_position_presence_matrix_fom_file(self, alignment_file, output_file, format="fasta", gap_symbol="-",
                                              verbose=True):

        alignment = AlignIO.read(alignment_file, format=format)

        position_matrix = self.get_position_presence_matrix(alignment, gap_symbol, verbose=verbose)
        np.savetxt(output_file, position_matrix, fmt="%i", delimiter='\t')
        #print position_matrix
        return position_matrix

    def count_unique_positions_per_sequence(self, alignment, gap_symbol="-", verbose=True, ):
        number_of_sequences = len(alignment)
        alignment_length = len(alignment[0])
        position_presence_matrix = self.get_position_presence_matrix(alignment, gap_symbol=gap_symbol, verbose=verbose)
        unique_position_count_dict = OrderedDict()

        for row in range(0, number_of_sequences):
            sequence_id = alignment[row].id
            unique_positions = 0
            for column in range(0, alignment_length):
                if (position_presence_matrix[row, column] == 1) or (position_presence_matrix[row, column] == -1):
                    unique_positions += 1

            unique_position_count_dict[sequence_id] = unique_positions

        return unique_position_count_dict

    def count_unique_positions_per_sequence_from_file(self, alignment_file, output_prefix, format="fasta",
                                                      gap_symbol="-", return_mode="absolute", verbose=True):

        alignment = AlignIO.read(alignment_file, format=format)
        number_of_sequences = len(alignment)
        alignment_length = len(alignment[0])
        position_presence_matrix = self.get_position_presence_matrix(alignment, gap_symbol=gap_symbol, verbose=verbose)
        unique_position_count_dict = SynDict()
        unique_position_count_percent_dict = SynDict()

        for row in range(0, number_of_sequences):
            sequence_id = alignment[row].id
            unique_positions = 0
            for column in range(0, alignment_length):
                if (position_presence_matrix[row, column] == 1) or (position_presence_matrix[row, column] == -1):
                    unique_positions += 1

            unique_position_count_dict[sequence_id] = unique_positions
            unique_position_count_percent_dict[sequence_id] = 100 * float(unique_positions) / (alignment_length - str(alignment[row].seq).count(gap_symbol))

        unique_position_count_dict.write("%s.absolute_counts" % output_prefix)
        unique_position_count_percent_dict.write("%s.percent_counts" % output_prefix)

        return unique_position_count_dict if return_mode == "absolute" else unique_position_count_percent_dict

    @staticmethod
    def parse_alignment(input_file, filetype="fasta"):
        print("Parsing alignment file %s" % input_file)
        #print(input_file)
        alignment = AlignIO.read(input_file, filetype)
        #print(alignment)
        print("Totaly %i sequences in alignment" % len(alignment))
        return alignment

    @staticmethod
    def make_regions_from_columns_to_retain(columns_to_retain,):
        region_coordinates_list = [[columns_to_retain[0], columns_to_retain[0] + 1]]

        if len(columns_to_retain) == 1:
            return region_coordinates_list

        for i in range(1, len(columns_to_retain)):
            if region_coordinates_list[-1][1] == columns_to_retain[i]:
                region_coordinates_list[-1][1] += 1
            else:
                region_coordinates_list.append([columns_to_retain[i], columns_to_retain[i] + 1])

        return region_coordinates_list

    def remove_columns_with_gaps(self, alignment, maximum_number_of_gaps_in_column, gap_symbol="-", verbose=False):
        num_of_sequences, len_of_alignment = self.get_general_statistics(alignment, verbose=verbose)
        if maximum_number_of_gaps_in_column > num_of_sequences:
            raise ValueError("Allowed number of sequences with gap is bigger than number of sequences")
        if maximum_number_of_gaps_in_column == num_of_sequences:
            if verbose:
                print("Alignment was not filtered: allowed number of sequence with gaps is equal to number of sequences")
            return alignment

        columns_without_gaps_list = []
        for i in range(0, len_of_alignment):
            if alignment[:, i].count(gap_symbol) <= maximum_number_of_gaps_in_column:
                columns_without_gaps_list.append(i)

        region_to_retain = self.make_regions_from_columns_to_retain(columns_without_gaps_list)
        filtered_alignment = alignment[:, region_to_retain[0][0]: region_to_retain[0][1]]
        if len(region_to_retain) == 1:
            return filtered_alignment

        for start, end in region_to_retain[1:]:
            filtered_alignment += alignment[:, start: end]

        return filtered_alignment

    @staticmethod
    def slice_fasta_alignment(alignment_file, output_file, start, end):
        alignment = AlignIO.read(alignment_file, format="fasta")
        SeqIO.write(alignment[start:end], output_file, "fasta")

    @staticmethod
    def prepare_multigene_alignment_for_codeml(alignment_file, coordinates_file, output_file,
                                               format="fasta"):
        sequence_dict = SeqIO.to_dict(SeqIO.parse(alignment_file, format=format))
        number_of_sequences = len(sequence_dict)
        alignment_length = len(sequence_dict[sequence_dict.keys()[0]])

        gene_lengths_in_codons = np.loadtxt(coordinates_file, dtype=int, comments='#', usecols=(0,))/3
        number_of_genes = len(gene_lengths_in_codons)
        codon_number_string = " ".join(map(str, gene_lengths_in_codons))
        with open(output_file, "w") as out_fd:
            out_fd.write("%i %i G\n" % (number_of_sequences, alignment_length))
            out_fd.write("G %i %s\n" % (number_of_genes, codon_number_string))
            for record_id in sequence_dict:
                out_fd.write("%s\n%s\n" % (record_id, str(sequence_dict[record_id].seq)))

    @staticmethod
    def get_db_ids(search_dict):
        id_set = set()
        for query_id in search_dict:
            for hit in search_dict[query_id]:
                id_set.add(hit.id)
        return id_set

    @staticmethod
    def get_codon_alignment(protein_alignment, nucleotide_seq_dict, codon_alignment_file,
                            protein2cds_accordance_dict=None):
        codon_alignment = {}
        if protein2cds_accordance_dict:
            for record in protein_alignment:
                if record.id not in protein2cds_accordance_dict:
                    print("Corresponding CDS was not found for %s protein" % record.id)
                    return -1
        for record in protein_alignment:
            nucleotide_seq = ""
            i = 0
            for aminoacid in record.seq:
                if aminoacid == "-":
                    nucleotide_seq += "---"
                    continue
                else:
                    nucleotide_seq += str(nucleotide_seq_dict[protein2cds_accordance_dict[record.id] if protein2cds_accordance_dict else record.id].seq[3*i:3*(i+1)])
                    i += 1
            codon_alignment[record.id] = SeqRecord(Seq(nucleotide_seq),
                                                   id=record.id,
                                                   description=record.description,
                                                   name=record.name)
            #print(record.id, record.seq)
        SeqIO.write(list(codon_alignment.values()), codon_alignment_file, "fasta")
        return codon_alignment

    def get_codon_alignment_from_files(self, protein_aln_file, nucleotide_seq_file, codon_alignment_file,
                                       cds2protein_accordance_file=None,
                                       alignment_format="fasta", nucleotide_sequence_format="fasta",
                                       cds_index_file=None, retain_cds_index=False):
        protein_aln_dict = AlignIO.read(protein_aln_file, format=alignment_format)
        nucleotide_seq_dict = SeqIO.index_db(cds_index_file if cds_index_file else "nuc_tmp.idx", nucleotide_seq_file,
                                             format=nucleotide_sequence_format)

        protein2cds_accordance_dict = None
        if cds2protein_accordance_file:
            protein2cds_accordance_dict = SynDict()
            protein2cds_accordance_dict.read(cds2protein_accordance_file, key_index=1, value_index=0)

        self.get_codon_alignment(protein_aln_dict, nucleotide_seq_dict, codon_alignment_file,
                                 protein2cds_accordance_dict=protein2cds_accordance_dict)
        if (not cds_index_file) and (not retain_cds_index):
            os.remove("nuc_tmp.idx")

    @staticmethod
    def merge_alignment(alignment_file_list, merged_alignment_file, coordinates_file, format="fasta"):
        #print("Merging alignments...")
        alignment_list = []
        sequence_lengthes = []
        #print(alignment_file_list)

        alignment_file_list_sorted = sorted(alignment_file_list)
        #print(alignment_file_list_sorted)
        for alignment_file in alignment_file_list_sorted:
            #alignment_file.sort()
            parsed = AlignIO.read(alignment_file, format=format)
            parsed.sort()
            alignment_list.append(parsed)
        merged_alignment = None
        for alignment in alignment_list:
            if not merged_alignment:
                sequence_lengthes.append(alignment.get_alignment_length())
                merged_alignment = alignment
                continue
            #print(alignment)
            sequence_lengthes.append(alignment.get_alignment_length())
            merged_alignment += alignment
        SeqIO.write(merged_alignment, merged_alignment_file, "fasta")
        sequence_coordinates = []
        #
        for seq_length in sequence_lengthes:
            if not sequence_coordinates:
                sequence_coordinates.append((1, seq_length))
                continue
            sequence_coordinates.append((sequence_coordinates[-1][1]+1, sequence_coordinates[-1][1]+seq_length))
        #print(sequence_coordinates)
        with open(coordinates_file, "w") as coord_fd:
            coord_fd.write("#length\tstart\tend\n")
            for coord_tuple in sequence_coordinates:
                coord_fd.write("%i\t%i\t%i\n" % (coord_tuple[1] - coord_tuple[0] + 1, coord_tuple[0], coord_tuple[1]))
        return merged_alignment, sequence_lengthes, sequence_coordinates

    def extract_degenerate_sites_from_codon_alignment(self, alignment, genetic_code_table=1,
                                                      remove_codon_columns_with_Ns=False):
        degenerate_codon_set = self.get_degenerate_codon_set(genetic_code_table)
        number_of_alignments = len(alignment)
        alignment_length = len(alignment[0])
        if alignment_length % 3 > 0:
            raise(ValueError, "Length of alignment is not divisible by 3")
        else:
            number_of_codons = int(alignment_length / 3)
        degenerate_columns = []
        degenerate_codons = []
        for i in range(0, number_of_codons):
            position_strings = []
            for j in range(0, 3):
                position_strings.append(list(set(alignment[:, 3*i + j])))
            if remove_codon_columns_with_Ns:
                if ("N" in position_strings[0]) or ("N" in position_strings[1]) or ("N" in position_strings[2]):
                    continue
            if (len(position_strings[0]) > 1) or (len(position_strings[1]) > 1):
                continue
            ambigious_codon = position_strings[0][0] + position_strings[1][0] + "N"
            """
            if Seq(ambigious_codon).translate(table=genetic_code_table) == "X":
                continue
            else:
                degenerate_columns.append(alignment[:, 3*i + 2])
                #print(i*3 +3)
            """
            if ambigious_codon in degenerate_codon_set:
                degenerate_columns.append(alignment[:, 3*i + 2])
                for k in 0, 1, 2:
                    degenerate_codons.append(alignment[:, 3*i + k])
        number_of_degenerate_columns = len(degenerate_columns)
        record_list = []
        degenerate_codons_record_list = []
        for i in range(0, number_of_alignments):
            string = ""
            for j in range(0, number_of_degenerate_columns):
                string += degenerate_columns[j][i]
            record = SeqRecord(seq=Seq(string), id=alignment[i].id, description=alignment[i].description)
            record_list.append(record)
            string = ""
            for j in range(0, 3 *number_of_degenerate_columns):
                string += degenerate_codons[j][i]
            record = SeqRecord(seq=Seq(string), id=alignment[i].id, description=alignment[i].description)
            degenerate_codons_record_list.append(record)


        #print(number_of_alignments, alignment_length)
        degenerate_alignment = MultipleSeqAlignment(record_list)
        degenerate_codons_alignment = MultipleSeqAlignment(degenerate_codons_record_list)
        return degenerate_alignment, degenerate_codons_alignment

    def extract_sites_by_column_expression_from_alignment(self, alignment, expression, remove_columns_with_Ns=False,
                                                          remove_columns_with_ambigous_nucleotides=False):
        number_of_alignments = len(alignment)
        alignment_length = len(alignment[0])
        extracted_columns = []

        for i in range(0, alignment_length):
            column_string = alignment[:, i]
            if remove_columns_with_Ns and ("N" in column_string):
                continue

            if remove_columns_with_ambigous_nucleotides:
                nucleotide_set = {s for s in column_string}
                if nucleotide_set - self.nucleotide_set:
                    continue
            if not expression(column_string):
                continue

            extracted_columns.append(column_string)

        number_of_extracted_columns = len(extracted_columns)
        record_list = []

        for i in range(0, number_of_alignments):
            string = ""
            for j in range(0, number_of_extracted_columns):
                string += extracted_columns[j][i]
            record = SeqRecord(seq=Seq(string), id=alignment[i].id, description=alignment[i].description)
            record_list.append(record)

        #print(number_of_alignments, alignment_length)
        extracted_alignment = MultipleSeqAlignment(record_list)

        return extracted_alignment

    def variable_alignment_column(self, column_string):
        different_nucleotide_number = 0

        for nucleotide in self.strict_nucleotide_list:
            self.tmp_count_dict[nucleotide] = column_string.count(nucleotide)
            if self.tmp_count_dict[nucleotide] > 0:
                different_nucleotide_number += 1

        return True if different_nucleotide_number > 1 else False

    def parsimony_informative_column(self, column_string):
        nucleotides_with_at_least_two_counts = 0

        for nucleotide in self.strict_nucleotide_list:
            self.tmp_count_dict[nucleotide] = column_string.count(nucleotide)
            if self.tmp_count_dict[nucleotide] >= 2:
                nucleotides_with_at_least_two_counts += 1

        return True if nucleotides_with_at_least_two_counts >= 2 else False

    def extract_variable_sites_from_alignment(self,
                                              alignment,
                                              remove_columns_with_Ns=False,
                                              remove_columns_with_ambigous_nucleotides=False):

        return self.extract_sites_by_column_expression_from_alignment(alignment,
                                                                      self.variable_alignment_column,
                                                                      remove_columns_with_Ns=remove_columns_with_Ns,
                                                                      remove_columns_with_ambigous_nucleotides=remove_columns_with_ambigous_nucleotides)

    def extract_variable_sites_from_alignment_from_file(self,
                                                        alignment_file,
                                                        output_file,
                                                        format="fasta",
                                                        remove_columns_with_Ns=False,
                                                        remove_columns_with_ambigous_nucleotides=False):

        alignment = AlignIO.read(alignment_file, format=format)
        variable_sites_alignment = self.extract_variable_sites_from_alignment(alignment,
                                                                              remove_columns_with_Ns=remove_columns_with_Ns,
                                                                              remove_columns_with_ambigous_nucleotides=remove_columns_with_ambigous_nucleotides)

        AlignIO.write([variable_sites_alignment], output_file, format=format)

    def extract_parsimony_informative_sites_from_alignment(self,
                                                           alignment,
                                                           remove_columns_with_Ns=False,
                                                           remove_columns_with_ambigous_nucleotides=False):

        return self.extract_sites_by_column_expression_from_alignment(alignment,
                                                                      self.parsimony_informative_column,
                                                                      remove_columns_with_Ns=remove_columns_with_Ns,
                                                                      remove_columns_with_ambigous_nucleotides=remove_columns_with_ambigous_nucleotides)

    def extract_parsimony_informative_sites_from_alignment_from_file(self,
                                                                     alignment_file,
                                                                     output_file,
                                                                     format="fasta",
                                                                     remove_columns_with_Ns=False,
                                                                     remove_columns_with_ambigous_nucleotides=False):

        alignment = AlignIO.read(alignment_file, format=format)
        variable_sites_alignment = self.extract_parsimony_informative_sites_from_alignment(alignment,
                                                                                           remove_columns_with_Ns=remove_columns_with_Ns,
                                                                                           remove_columns_with_ambigous_nucleotides=remove_columns_with_ambigous_nucleotides)

        AlignIO.write([variable_sites_alignment], output_file, format=format)

    def extract_degenerate_sites_from_codon_alignment_from_file(self, alignment_file, output_prefix,
                                                                genetic_code_table=1, format="fasta",
                                                                remove_codon_columns_with_Ns=False):
        alignment = AlignIO.read(alignment_file, format=format)
        degenerated_alignment, degenerate_codons_alignment = self.extract_degenerate_sites_from_codon_alignment(alignment,
                                                                                                                genetic_code_table=genetic_code_table,
                                                                                                                remove_codon_columns_with_Ns=remove_codon_columns_with_Ns)

        output_alignment_file = "%s.4fold_degenerated_sites.fasta" % output_prefix
        output_codon_alignment_file = "%s.codons_with_4fold_degenerated_sites.fasta" % output_prefix
        AlignIO.write([degenerated_alignment], output_alignment_file, format=format)
        AlignIO.write([degenerate_codons_alignment], output_codon_alignment_file, format=format)


    @staticmethod
    def sequences_from_alignment_generator(alignments, gap_symbol="-"):
        for record in alignments:
            record.seq = Seq(str(record.seq).replace(gap_symbol, ""))
            yield record

    def extract_sequences_from_alignment(self, alignment_file, output_file, alignment_format="fasta",
                                         output_format="fasta", gap_symbol="-"):
        alignments = AlignIO.read(alignment_file, format=alignment_format)
        SeqIO.write(self.sequences_from_alignment_generator(alignments, gap_symbol=gap_symbol),
                    output_file, format=output_format)

    @staticmethod
    def translate_codon_alignment(codon_alignment_file, protein_alignment_file, format="fasta",
                                  gap_symbol="-", table=1):
        alignment = AlignIO.read(codon_alignment_file, format=format)

        def translated_record_generator(alignment):
            for record in alignment:
                yield SeqRecord(seq=record.seq.translate(gap=gap_symbol, table=table), id=record.id,
                                description=record.description)

        SeqIO.write(translated_record_generator(alignment), protein_alignment_file, format=format)

    def gather_alignment_stats(self, alignment, gap_symbol="-", verbose=False):

        number_of_sequences, alignment_length = self.get_general_statistics(alignment, verbose=verbose)

        unique_position_count_dict = self.count_unique_positions_per_sequence(alignment, gap_symbol=gap_symbol,
                                                                              verbose=verbose)
    @staticmethod
    def get_seq_id_dict(alignment):
        seq_id_index_dict = SynDict()
        for i in range(0, len(alignment)):
            seq_id_index_dict[i] = alignment[i].id
        return seq_id_index_dict

    def count_dNdS_by_reference_seq_in_codon_alignment(self, codon_alignment, reference_seq_id, genetic_code_table=1,
                                                       gap_symbol_list=["-",], use_ambigious_table=False,
                                                       output_file=None):

        number_of_sequences = len(codon_alignment)
        alignment_length = len(codon_alignment[0])
        if alignment_length % 3 > 0:
            raise ValueError("Length of alignment is not divisible by 3")
        else:
            number_of_codons = int(alignment_length / 3)

        seq_id_index_dict = self.get_seq_id_dict(codon_alignment)
        for i in range(0, number_of_sequences):
            if seq_id_index_dict[i] == reference_seq_id:
                reference_seq_index = i
                break
        else:
            raise ValueError("Reference sequence id (%s) is absent in alignment" % reference_seq_id)

        translation_table = CodonTable.ambiguous_dna_by_id[genetic_code_table] if use_ambigious_table else CodonTable.unambiguous_dna_by_id[genetic_code_table]
        dN_dS_W_dict = TwoLvlDict()

        for i in range(0, number_of_sequences):
            if i != reference_seq_index:
                dN_dS_W_dict[seq_id_index_dict[i]] = OrderedDict({"dN": 0, "dS": 0, "W": 'NA'})

        for i in range(0, number_of_codons):
            reference_codon = codon_alignment[reference_seq_index, i*3: (i+1) * 3]
            for symbol in gap_symbol_list:
                if symbol in reference_codon:
                    break
            else:
                reference_aminoacid = translation_table.forward_table[reference_codon]
                if reference_aminoacid == "X":
                    continue
                for j in range(0, number_of_sequences):
                    if j == reference_seq_index:
                        continue
                    sequence_codon = codon_alignment[j, i * 3: (i + 1) * 3]
                    for symbol in gap_symbol_list:
                        if symbol in sequence_codon:
                            break
                    else:
                        sequence_aminoacid = translation_table.forward_table[sequence_codon]
                        if sequence_aminoacid == "X":
                            continue

                        if (sequence_aminoacid == reference_aminoacid) and (sequence_codon != reference_codon):
                            dN_dS_W_dict[seq_id_index_dict[j]]["dS"] += 1
                        elif sequence_aminoacid != reference_aminoacid:
                            dN_dS_W_dict[seq_id_index_dict[j]]["dN"] += 1

        for sequence_id in dN_dS_W_dict:
            if dN_dS_W_dict[sequence_id]["dS"] != 0:
                dN_dS_W_dict[sequence_id]["W"] = float(dN_dS_W_dict[sequence_id]["dN"]) / float(dN_dS_W_dict[sequence_id]["dS"])

        if output_file:
            dN_dS_W_dict.write(output_file, absent_symbol="NA")

        return dN_dS_W_dict

    def count_dNdS_by_reference_seq_in_codon_alignment_from_file(self, alignment_file, reference_seq_id,
                                                                 genetic_code_table=1, gap_symbol_list=["-",],
                                                                 use_ambigious_table=False, output_file=None,
                                                                 format="fasta"):

        codon_alignment = self.parse_alignment(alignment_file, filetype=format)

        return self.count_dNdS_by_reference_seq_in_codon_alignment(codon_alignment, reference_seq_id,
                                                                   genetic_code_table=genetic_code_table,
                                                                   gap_symbol_list=gap_symbol_list,
                                                                   use_ambigious_table=use_ambigious_table,
                                                                   output_file=output_file)

    @staticmethod
    def extract_codon_positions(alignment):

        return [alignment[:, i::3] for i in (0, 1, 2)]

    def extract_codon_positions_from_file(self, alignment_file, output_prefix, format="fasta"):
        alignment = self.parse_alignment(alignment_file, filetype="fasta")

        codon_position_alignments = self.extract_codon_positions(alignment)

        for position in 0, 1, 2:
            output_file = "%s.pos_%i%s" % (output_prefix, position + 1, self.split_filename(alignment_file)[-1])
            AlignIO.write(codon_position_alignments[position], output_file, format=format)

    @staticmethod
    def get_position_correspondence_matrix(alignment, gap_symbol="-", verbose=True):

        number_of_sequences = len(alignment)
        alignment_length = len(alignment[0])
        # converting alignment to numpy letter array stored by columns!
        alignment_array = np.array([list(rec) for rec in alignment], np.character, order="F")

        if verbose:
            print("%i sequences in alignment" % number_of_sequences)
            print("%i columns in alignment" % alignment_length)

        position_correspondence_array = np.array([[0 for letter in rec.seq] for rec in alignment], int, order="F")

        current_row_index_list = [0 for rec in alignment]
        for column in range(0, alignment_length):
            for row in range(0, number_of_sequences):
                if alignment_array[row, column] == gap_symbol:
                    position_correspondence_array[row, column] = -1

                else:
                    position_correspondence_array[row, column] = current_row_index_list[row]
                    current_row_index_list[row] += 1

        #for row in range(0, number_of_sequences):
        #    for column in range(0, alignment_length):
        #print alignment_array[0, ]
        #print alignment_array[:, 1]
        #print alignment_array
        #print position_presence_array
        return position_correspondence_array

    def get_position_correspondence_matrix_fom_file(self, alignment_file, output_file, format="fasta", gap_symbol="-",
                                                    verbose=True):

        alignment = AlignIO.read(alignment_file, format=format)

        position_matrix = self.get_position_correspondence_matrix(alignment, gap_symbol, verbose=verbose)
        np.savetxt(output_file, position_matrix, fmt="%i", delimiter='\t')
        #print position_matrix
        return position_matrix

    def get_specific_positions(self, alignment_file, reference_sequence_id, reference_position_list, output_prefix,
                               format="fasta", gap_symbol="-", verbose=True, alignment_type="nucleotide", flank_length=0):
        alignment = AlignIO.read(alignment_file, format=format)

        alignment_length = len(alignment[0].seq)
        record_id_list = [record.id for record in alignment]
        record_number = len(alignment)

        record_seq_nogaps_list = [record.seq.ungap(gap=gap_symbol) for record in alignment]
        sequence_len_list = [len(record_seq) for record_seq in record_seq_nogaps_list]

        position_list = [reference_position_list] if isinstance(reference_position_list, int) else reference_position_list
        position_list.sort()
        position_list = np.array(position_list) - 1 # convertion to 0-based
        print(position_list)

        if alignment_type == "codon":
            position_list *= 3

        position_matrix = self.get_position_correspondence_matrix(alignment, gap_symbol, verbose=verbose)
        np.savetxt("%s.position_matrix" % output_prefix, position_matrix, fmt="%i", delimiter='\t')

        reference_sequence_index = 0
        for record in alignment:
            if record.id == reference_sequence_id:
                break
            reference_sequence_index += 1

        alignment_position_list = []
        prev_ref_pos_col = 0
        for pos in position_list:
            for col_index in range(prev_ref_pos_col, alignment_length):
                if position_matrix[reference_sequence_index, col_index] == pos:
                    prev_ref_pos_col = col_index
                    alignment_position_list.append(col_index)
                    break

        header = "#alignment_pos"
        for record_id in record_id_list:
            header += "\t%s" % record_id
        for record_id in record_id_list:
            header += "\t%s,%s" % (record_id, alignment_type)
        if flank_length > 0:
            for record_id in record_id_list:
                for flank_pos in "upstream", "downstream":
                    header += "\t%s,%s" % (record_id, flank_pos)
        if flank_length > 0:
            for record_id in record_id_list:
                for flank_pos in "upstream", "downstream":
                    header += "\t%s,%s(no_gaps)" % (record_id, flank_pos)
        header += "\n"

        results_list = []

        for alignment_pos in alignment_position_list:

            results = [alignment_pos / 3 + 1] if alignment_type == "codon" else [alignment_pos + 1]  # conversion to 1-based
            for row_index in range(0, record_number):
                sequence_pos_list = [(pos/3 + 1) if alignment_type == "codon" else (pos + 1) for pos in position_matrix[:, alignment_pos]] # conversion to 1-based
            alignment_nucleotide_list = []
            flank_list = []
            flank_list_no_gaps = []
            if alignment_type == "codon":
                for record_index in range(0, record_number):
                    alignment_nucleotide_list.append(str(alignment[record_index, alignment_pos:alignment_pos+3].seq))
            else:
                alignment_nucleotide_list = [letter for letter in alignment[:, alignment_pos]]
            if flank_length > 0:
                for record_index in range(0, record_number):
                    sequence_pos = position_matrix[record_index, alignment_pos]

                    # upstream flank extraction
                    flank_list.append(str(alignment[record_index, max((0, alignment_pos - flank_length)):alignment_pos].seq))
                    #print record_seq_nogaps_list[record_index]
                    if sequence_pos == -1: # empty flanks with no_gaps if position is in gap
                        flank_list_no_gaps.append("")
                    else:
                        flank_list_no_gaps.append(str(record_seq_nogaps_list[record_index][max((0, sequence_pos - flank_length)):sequence_pos]))
                    #downstream flank extraction
                    if alignment_type == "codon":
                        flank_list.append(str(alignment[record_index, alignment_pos + 3:min(alignment_length, alignment_pos + 3 + flank_length)].seq))
                        if sequence_pos == -1: # empty flanks with no_gaps if position is in gap
                            flank_list_no_gaps.append("")
                        else:
                            flank_list_no_gaps.append(str(record_seq_nogaps_list[record_index][sequence_pos + 3:min(sequence_len_list[record_index], sequence_pos + 3 + flank_length)]))
                    else:
                        flank_list.append(str(alignment[record_index, alignment_pos + 1:min(alignment_length, alignment_pos + 1 + flank_length)].seq))
                        if sequence_pos == -1: # empty flanks with no_gaps if position is in gap
                            flank_list_no_gaps.append("")
                        else:
                            flank_list_no_gaps.append(str(record_seq_nogaps_list[record_index][sequence_pos + 1:min(sequence_len_list[record_index], sequence_pos + 1 + flank_length)]))
            results_list.append(results + sequence_pos_list + alignment_nucleotide_list + flank_list + flank_list_no_gaps)
            #print "\n"
        with open("%s.output" % output_prefix, "w") as out_fd:
            out_fd.write(header)
            for entry in results_list:
                out_fd.write("\t".join(map(str, entry)) + "\n")

        return results_list

    def get_specific_positions_for_multiple_files(self, alignment_dir, position_file, reference_sequence_id, output_dir, alignment_file_suffix="",
                                                  format="fasta", gap_symbol="-", verbose=True, alignment_type="nucleotide", flank_length=0):

        position_dict = SynDict(filename=position_file, split_values=True, expression=int)

        self.safe_mkdir(output_dir)

        file_list = ["%s/%s" % (alignment_dir, filename) for filename in os.listdir(alignment_dir)]

        for alignment_name in position_dict:
            alignment_file = "%s/%s%s" % (alignment_dir, alignment_name, alignment_file_suffix)
            output_prefix = "%s/%s" % (output_dir, alignment_name)
            if alignment_file not in file_list:
                print("WARNING!!! No file for %s. Skipping..." % alignment_name)
                break
            self.get_specific_positions(alignment_file, reference_sequence_id, position_dict[alignment_name],
                                        output_prefix, format=format, gap_symbol=gap_symbol, verbose=verbose,
                                        alignment_type=alignment_type, flank_length=flank_length)

    def call_variants_from_multiple_alignment(self, alignment, reference_sequence_id, gap_symbol="-",
                                              verbose=True, output_prefix=None, align_variants=False,
                                              output_type="hgvs", variant_separator=",", target_sequence_id=None,
                                              absent_symbol=""):
        alignment_length = len(alignment[0].seq)
        record_id_list = [record.id for record in alignment]
        record_number = len(alignment)

        record_seq_nogaps_list = [record.seq.ungap(gap=gap_symbol) for record in alignment]
        sequence_len_list = [len(record_seq) for record_seq in record_seq_nogaps_list]

        reference_sequence_index = 0
        for record in alignment:
            if record.id == reference_sequence_id:
                break
            reference_sequence_index += 1

        pos_cor_matrix = self.get_position_correspondence_matrix(alignment, gap_symbol=gap_symbol, verbose=verbose)
        alignment_array = np.array([list(rec) for rec in alignment], np.character, order="F")

        variant_dict = OrderedDict()

        for record_index in range(0, record_number):
            if record_index == reference_sequence_index:
                continue
            variant_dict[record_id_list[record_index]] = []
            pos_index = 0

            while pos_index < alignment_length:
                if alignment_array[record_index, pos_index] == alignment_array[reference_sequence_index, pos_index]:
                    pos_index += 1
                    continue

                if (alignment_array[record_index, pos_index] == gap_symbol) or (alignment_array[reference_sequence_index, pos_index] == gap_symbol):
                    # indel start
                    #print "aaa"
                    if alignment_array[reference_sequence_index, pos_index] == gap_symbol and pos_index > 0:

                        indel_start_pos = pos_index -1
                    else:
                        indel_start_pos = pos_index

                    if alignment_array[record_index, pos_index] == gap_symbol:
                        deletion_length = 1
                        insertion_length = 0
                        deletion_seq = alignment_array[reference_sequence_index, pos_index]
                        insertion_seq = ""
                    else:
                        deletion_length = 0
                        insertion_length = 1
                        deletion_seq = ""
                        insertion_seq = alignment_array[record_index, pos_index]
                        
                    pos_index += 1

                    while pos_index < alignment_length:

                        if (alignment_array[record_index, pos_index] == gap_symbol) and \
                           (alignment_array[reference_sequence_index, pos_index] == gap_symbol):
                            pass
                        elif alignment_array[record_index, pos_index] == gap_symbol:
                            deletion_length += 1
                            deletion_seq += alignment_array[reference_sequence_index, pos_index]

                        elif alignment_array[reference_sequence_index, pos_index] == gap_symbol:
                            insertion_length += 1
                            insertion_seq += alignment_array[record_index, pos_index]

                        else:
                            break
                        pos_index += 1

                    if deletion_length + insertion_length == 1:
                        indel_end_ref_coord = None
                    else:
                        indel_end_ref_coord = pos_cor_matrix[reference_sequence_index, pos_index - 1] + 1

                    if pos_cor_matrix[reference_sequence_index, indel_start_pos] > -1:
                        #print pos_cor_matrix[reference_sequence_index, indel_start_pos]
                        indel_start_ref_coord = pos_cor_matrix[reference_sequence_index, indel_start_pos] + 1
                    else:
                        #print pos_cor_matrix[reference_sequence_index, indel_start_pos]
                        #print indel_start_pos
                        indel_start_ref_coord = -1


                    variant_type = 'del' if insertion_length == 0 else 'ins' if deletion_length == 0 else 'delins'
                    variant_dict[record_id_list[record_index]].append([variant_type,
                                                                       indel_start_ref_coord,
                                                                       None if variant_type == 'ins' else indel_end_ref_coord,
                                                                       deletion_seq if deletion_length > 0 else None,
                                                                       insertion_seq if insertion_length > 0 else None])
                    #print variant_dict[record_id_list[record_index]][-1]
                else:
                    variant_dict[record_id_list[record_index]].append(['snp',
                                                                       pos_cor_matrix[reference_sequence_index, pos_index] + 1,
                                                                       None,
                                                                       alignment_array[reference_sequence_index, pos_index],
                                                                       alignment_array[record_index, pos_index]])
                    pos_index += 1

        #print variant_dict
        if output_prefix and (output_type == 'hgvs'):
            with self.metaopen("%s.variants.tab" % output_prefix, "w") as out_fd:
                out_fd.write("#seq_id\tvariants\n")
                record_string_dict = OrderedDict()
                if align_variants:
                    variant_start_coordinates = []
                    variant_count_dict = OrderedDict()
                    for record_id in variant_dict:

                        for variant_entry in variant_dict[record_id]:
                            variant_start_coordinates.append(variant_entry[1])

                        variant_count_dict[record_id] = len(variant_dict[record_id])
                    variant_start_coordinates = list(set(variant_start_coordinates))
                    variant_start_coordinates.sort()
                    print("%s variants with different start coordinates" % len(variant_start_coordinates))
                    for record_id in variant_count_dict:
                        print("%s: %i" % (record_id, variant_count_dict[record_id]))
                        #print variant_dict[record_id]

                    for record_id in variant_dict:
                        variant_list = []
                        variant_index = 0
                        for i in range(0, len(variant_start_coordinates)):
                            if variant_index < variant_count_dict[record_id] and (variant_start_coordinates[i] == variant_dict[record_id][variant_index][1]):
                                variant_list.append(self.hgvsall_from_variant_list_entry(variant_dict[record_id][variant_index]))
                                variant_index += 1
                            else:
                                #print "bbbb"
                                #print variant_start_coordinates[i] ,  variant_dict[record_id][variant_index][1]
                                variant_list.append(np.NaN)
                        record_string_dict[record_id] = variant_list

                    record_df = pd.DataFrame.from_dict(record_string_dict)
                    record_df.to_csv("%s.all.variants" % output_prefix, sep="\t", index=False,
                                     header=True, na_rep=absent_symbol)

                    if target_sequence_id:
                        boolean_comparison_array = ~pd.isnull(record_df[target_sequence_id])
                        for record_id in record_id_list:
                            if (record_id == reference_sequence_id) or (record_id == target_sequence_id):
                                continue
                            boolean_comparison_array &= np.not_equal(record_df[target_sequence_id],
                                                                     record_df[record_id]) #numexpr.evaluate('(a==b)|((a!=a)&(b!=b))') &=

                        filtered_df = record_df[boolean_comparison_array]
                        filtered_df.to_csv("%s.target_specific.variants" % output_prefix, sep="\t", index=False,
                                           header=True, na_rep=absent_symbol)
                    #print (record_df)
                    #print (record_df["mustela_nigripes"][[220, 225]])
                    #print ~pd.isnull(record_df["mustela_nigripes"]) &
                        print (filtered_df)
                else:
                    for record_id in variant_dict:
                        variant_list = []
                        for variant in variant_dict[record_id]:
                            variant_list.append(self.hgvsall_from_variant_list_entry(variant))
                        record_string_dict[record_id] = variant_list

                for record_id in record_string_dict:
                    record_string = "%s\t" % record_id
                    record_string += variant_separator.join(map(lambda s: "" if (s is None) or (s is np.NaN) else s,
                                                                record_string_dict[record_id]))
                    record_string += "\n"
                    out_fd.write(record_string)

        return variant_dict

    def hgvsall_from_variant_list_entry(self, variant):
        if variant[0] == 'snp':
            return '%i%sto%s' % (variant[1], variant[3], variant[4])
        elif variant[0] == 'delins':
            return '%i_%idel%sins%s' % (variant[1], variant[2], variant[3], variant[4])
        elif variant[0] == 'del':
            return '%i_%i%s%s' % (variant[1],
                                  variant[2],
                                  variant[0],
                                  variant[3] if variant[4] is None else variant[4])
        elif variant[0] == 'ins':
            return '%i%s%s' % (variant[1],
                               variant[0],
                               variant[3] if variant[4] is None else variant[4])
        else:
            raise ValueError("ERROR!!! Unrecognized variant type! %s" % str(variant))

    def call_variants_from_multiple_alignment_from_file(self,
                                                        alignment_file,
                                                        output_prefix,
                                                        reference_sequence_id,
                                                        gap_symbol="-",
                                                        verbose=True,
                                                        format="fasta",
                                                        align_variants=False,
                                                        output_type="hgvs",
                                                        variant_separator=",",
                                                        target_sequence_id=None,
                                                        absent_symbol=""):

        alignment_dict = AlignIO.read(alignment_file, format=format)
        self.call_variants_from_multiple_alignment(alignment_dict,
                                                   reference_sequence_id,
                                                   gap_symbol=gap_symbol,
                                                   verbose=verbose,
                                                   output_prefix=output_prefix,
                                                   align_variants=align_variants,
                                                   output_type=output_type,
                                                   variant_separator=variant_separator,
                                                   target_sequence_id=target_sequence_id,
                                                   absent_symbol=absent_symbol)




#!/usr/bin/env python

from collections import OrderedDict

GFF_COLS = OrderedDict({
                        "scaffold": 0,
                        "source":   1,
                        "featuretype": 2,
                        "start": 3,
                        "end": 4,
                        "score": 5,
                        "strand": 6,
                        "phase": 7,
                        "attributes": 8
                        })

BED_COLS = OrderedDict({
                        "scaffold": 0,
                        "start": 1,
                        "end": 2
                        })
BUSCO_TABLE_COLS = OrderedDict({
                                "id": 0,
                                "status": 1,
                                "contig": 2,
                                "start": 3,
                                "end": 4,
                                "score": 5,
                                "length": 6})



#!/usr/bin/env python
import os
from RouToolPa.Tools.Abstract import Tool


class CombineVariants4(Tool):

    def __init__(self, max_threads=4, max_memory=None, timelog=None):
        Tool.__init__(self,
                      "gatk --java-options -Xmx%s CombineVariants" % max_memory if max_memory else "gatk CombineVariants",
                      max_threads=max_threads, max_memory=max_memory,
                      timelog=timelog)
    # http://www.broadinstitute.org/gatk/gatkdocs/org_broadinstitute_sting_gatk_walkers_variantutils_CombineVariants.html

    def combine_from_same_source(self, reference_file, input_filelist, output_vcf):

        options = " -R %s" % reference_file
        options += " -nt %i" % self.threads
        options += " --variant %s" % (input_filelist if isinstance(input_filelist, str) else " --variant ".join(input_filelist))
        options += " --genotypemergeoption UNSORTED"
        options += " -O %s" % output_vcf

        self.execute(options=options)



#!/usr/bin/env python
import os
from RouToolPa.Tools.Abstract import Tool



class TVC(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "tvc", path=path, max_threads=max_threads)

    def variant_caller_pipeline(self, input_bam, reference_fasta, out_dir="./", hotspot_vcf=None, param_file=None,
                                tvc_root_dir=None, primer_trim_bed=None, postprocesed_bam=None, region_bed=None,
                                error_motif_file=None):
        options = ""
        options += " -b %s" % region_bed if region_bed else ""
        options += " -s %s" % hotspot_vcf if hotspot_vcf else ""
        options += " -i %s" % input_bam
        options += " -r %s" % reference_fasta
        options += " -o %s" % out_dir
        options += " -p %s" % param_file if param_file else ""
        options += " -B %s" % tvc_root_dir if tvc_root_dir else ""
        options += " -n %i" % self.threads
        options += " --primer-trim-bed=%s" % primer_trim_bed if primer_trim_bed else ""
        options += " --postprocessed-bam=%s" % postprocesed_bam if postprocesed_bam else ""


        self.execute(options, cmd="variant_caller_pipeline.py")

    def variant_call(self, input_bam, reference_fasta, out_vcf, out_dir="./", sample_name=None, num_variants_per_thread=None,
                     parameters_file=None, force_sample_name=None, target_file=None, trim_ampliseq_primers=None,
                     downsample_to_coverage=None, model_file=None, recal_model_hp_thres=None,
                     output_vcf=None, suppress_reference_genotypes=None, suppress_no_calls=None,
                     suppress_nocall_genotypes=None, allow_snps=None, allow_indels=None, allow_mnps=None,
                     allow_complex=None, max_complex_gap=None, use_best_n_alleles=None, min_mapping_qv=None,
                     read_snp_limit=None, read_max_mismatch_fraction=None, gen_min_alt_allele_freq=None,
                     gen_min_indel_alt_allele_freq=None, gen_min_coverage=None, input_vcf=None,
                     process_input_positions_only=None, use_input_allele_only=None, use_sse_basecaller=None,
                     resolve_clipped_bases=None, prediction_precision=None, shift_likelihood_penalty=None,
                     minimum_sigma_prior=None, slope_sigma_prior=None, sigma_prior_weight=None, k_zero=None,
                     sse_relative_safety_level=None, tune_sbias=None, snp_min_coverage=None,
                     snp_min_cov_each_strand=None, snp_min_variant_score=None, snp_strand_bias=None,
                     snp_strand_bias_pval=None, snp_min_allele_freq=None, mnp_min_coverage=None,
                     mnp_min_cov_each_strand=None, mnp_min_variant_score=None, mnp_strand_bias=None,
                     mnp_strand_bias_pval=None, mnp_min_allele_freq=None, indel_min_coverage=None,
                     indel_min_cov_each_strand=None, indel_min_variant_score=None, indel_strand_bias=None,
                     indel_strand_bias_pval=None, indel_min_allele_freq=None, hotspot_min_coverage=None,
                     hotspot_min_cov_each_strand=None, hotspot_min_variant_score=None, hotspot_strand_bias=None,
                     hotspot_strand_bias_pval=None, hotspot_min_allele_freq=None, hp_max_length=None,
                     error_motifs=None, sse_prob_threshold=None, min_ratio_reads_non_sse_strand=None,
                     data_quality_stringency=None, read_rejection_threshold=None, filter_unusual_predictions=None,
                     filter_deletion_predictions=None, filter_insertion_predictions=None, heal_snps=None,
                     min_delta_for_flow=None, max_flows_to_test=None, outlier_probability=None, heavy_tailed=None,
                     suppress_recalibration=None, do_snp_realignment=None, do_mnp_realignment=None):

        options = ""
        options += " --input-bam %s" % input_bam
        options += " --reference %s" % reference_fasta
        options += " --output-dir %s" % out_dir
        options += " --output-vcf %s" % out_vcf

        options += " --sample-name %s" % sample_name if sample_name else ""
        options += " --num-variants-per-thread %s" % str(num_variants_per_thread) if num_variants_per_thread else ""
        options += " --parameters-file %s" % str(parameters_file) if parameters_file else ""
        options += " --force-sample-name %s" % str(force_sample_name) if force_sample_name else ""
        options += " --target-file %s" % str(target_file) if target_file else ""
        options += " --trim-ampliseq-primers %s" % str(trim_ampliseq_primers) if trim_ampliseq_primers else ""
        options += " --downsample-to-coverage %s" % str(downsample_to_coverage) if downsample_to_coverage else ""
        options += " --model-file %s" % str(model_file) if model_file else ""
        options += " --recal-model-hp-thres %s" % str(recal_model_hp_thres) if recal_model_hp_thres else ""
        options += " --output-vcf %s" % str(output_vcf) if output_vcf else ""
        options += " --suppress-reference-genotypes %s" % str(suppress_reference_genotypes) if suppress_reference_genotypes else ""
        options += " --suppress-no-calls %s" % str(suppress_no_calls) if suppress_no_calls else ""
        options += " --suppress-nocall-genotypes %s" % str(suppress_nocall_genotypes) if suppress_nocall_genotypes else ""
        options += " --allow-snps %s" % str(allow_snps) if allow_snps else ""
        options += " --allow-indels %s" % str(allow_indels) if allow_indels else ""
        options += " --allow-mnps %s" % str(allow_mnps) if allow_mnps else ""
        options += " --allow-complex %s" % str(allow_complex) if allow_complex else ""
        options += " --max-complex-gap %s" % str(max_complex_gap) if max_complex_gap else ""
        options += " --use-best-n-alleles %s" % str(use_best_n_alleles) if use_best_n_alleles else ""
        options += " --min-mapping-qv %s" % str(min_mapping_qv) if min_mapping_qv else ""
        options += " --read-snp-limit %s" % str(read_snp_limit) if read_snp_limit else ""
        options += " --read-max-mismatch-fraction %s" % str(read_max_mismatch_fraction) if read_max_mismatch_fraction else ""
        options += " --gen-min-alt-allele-freq %s" % str(gen_min_alt_allele_freq) if gen_min_alt_allele_freq else ""
        options += " --gen-min-indel-alt-allele-freq %s" % str(gen_min_indel_alt_allele_freq) if gen_min_indel_alt_allele_freq else ""
        options += " --gen-min-coverage %s" % str(gen_min_coverage) if gen_min_coverage else ""
        options += " --input-vcf %s" % str(input_vcf) if input_vcf else ""
        options += " --process-input-positions-only %s" % str(process_input_positions_only) if process_input_positions_only else ""
        options += " --use-input-allele-only %s" % str(use_input_allele_only) if use_input_allele_only else ""
        options += " --min-delta-for-flow %s" % str(min_delta_for_flow) if min_delta_for_flow else ""
        options += " --max-flows-to-test %s" % str(max_flows_to_test) if max_flows_to_test else ""
        options += " --outlier-probability %s" % str(outlier_probability) if outlier_probability else ""
        options += " --heavy-tailed %s" % str(heavy_tailed) if heavy_tailed else ""
        options += " --suppress-recalibration %s" % str(suppress_recalibration) if suppress_recalibration else ""
        options += " --do-snp-realignment %s" % str(do_snp_realignment) if do_snp_realignment else ""
        options += " --do-mnp-realignment %s" % str(do_mnp_realignment) if do_mnp_realignment else ""
        options += " --use-sse-basecaller %s" % str(use_sse_basecaller) if use_sse_basecaller else ""
        options += " --resolve-clipped-bases %s" % str(resolve_clipped_bases) if resolve_clipped_bases else ""
        options += " --prediction-precision %s" % str(prediction_precision) if prediction_precision else ""
        options += " --shift-likelihood-penalty %s" % str(shift_likelihood_penalty) if shift_likelihood_penalty else ""
        options += " --minimum-sigma-prior %s" % str(minimum_sigma_prior) if minimum_sigma_prior else ""
        options += " --slope-sigma-prior %s" % str(slope_sigma_prior) if slope_sigma_prior else ""
        options += " --sigma-prior-weight %s" % str(sigma_prior_weight) if sigma_prior_weight else ""
        options += " --k-zero %s" % str(k_zero) if k_zero else ""
        options += " --sse-relative-safety-level %s" % str(sse_relative_safety_level) if sse_relative_safety_level else ""
        options += " --tune-sbias %s" % str(tune_sbias) if tune_sbias else ""
        options += " --snp-min-coverage %s" % str(snp_min_coverage) if snp_min_coverage else ""
        options += " --snp-min-cov-each-strand %s" % str(snp_min_cov_each_strand) if snp_min_cov_each_strand else ""
        options += " --snp-min-variant-score %s" % str(snp_min_variant_score) if snp_min_variant_score else ""
        options += " --snp-strand-bias %s" % str(snp_strand_bias) if snp_strand_bias else ""
        options += " --snp-strand-bias-pval %s" % str(snp_strand_bias_pval) if snp_strand_bias_pval else ""
        options += " --snp-min-allele-freq %s" % str(snp_min_allele_freq) if snp_min_allele_freq else ""
        options += " --mnp-min-coverage %s" % str(mnp_min_coverage) if mnp_min_coverage else ""
        options += " --mnp-min-cov-each-strand %s" % str(mnp_min_cov_each_strand) if mnp_min_cov_each_strand else ""
        options += " --mnp-min-variant-score %s" % str(mnp_min_variant_score) if mnp_min_variant_score else ""
        options += " --mnp-strand-bias %s" % str(mnp_strand_bias) if mnp_strand_bias else ""
        options += " --mnp-strand-bias-pval %s" % str(mnp_strand_bias_pval) if mnp_strand_bias_pval else ""
        options += " --mnp-min-allele-freq %s" % str(mnp_min_allele_freq) if mnp_min_allele_freq else ""
        options += " --indel-min-coverage %s" % str(indel_min_coverage) if indel_min_coverage else ""
        options += " --indel-min-cov-each-strand %s" % str(indel_min_cov_each_strand) if indel_min_cov_each_strand else ""
        options += " --indel-min-variant-score %s" % str(indel_min_variant_score) if indel_min_variant_score else ""
        options += " --indel-strand-bias %s" % str(indel_strand_bias) if indel_strand_bias else ""
        options += " --indel-strand-bias-pval %s" % str(indel_strand_bias_pval) if indel_strand_bias_pval else ""
        options += " --indel-min-allele-freq %s" % str(indel_min_allele_freq) if indel_min_allele_freq else ""
        options += " --hotspot-min-coverage %s" % str(hotspot_min_coverage) if hotspot_min_coverage else ""
        options += " --hotspot-min-cov-each-strand %s" % str(hotspot_min_cov_each_strand) if hotspot_min_cov_each_strand else ""
        options += " --hotspot-min-variant-score %s" % str(hotspot_min_variant_score) if hotspot_min_variant_score else ""
        options += " --hotspot-strand-bias %s" % str(hotspot_strand_bias) if hotspot_strand_bias else ""
        options += " --hotspot-strand-bias-pval %s" % str(hotspot_strand_bias_pval) if hotspot_strand_bias_pval else ""
        options += " --hotspot-min-allele-freq %s" % str(hotspot_min_allele_freq) if hotspot_min_allele_freq else ""
        options += " --hp-max-length %s" % str(hp_max_length) if hp_max_length else ""
        options += " --error-motifs %s" % str(error_motifs) if error_motifs else ""
        options += " --sse-prob-threshold %s" % str(sse_prob_threshold) if sse_prob_threshold else ""
        options += " --min-ratio-reads-non-sse-strand %s" % str(min_ratio_reads_non_sse_strand) if min_ratio_reads_non_sse_strand else ""
        options += " --data-quality-stringency %s" % str(data_quality_stringency) if data_quality_stringency else ""
        options += " --read-rejection-threshold %s" % str(read_rejection_threshold) if read_rejection_threshold else ""
        options += " --filter-unusual-predictions %s" % str(filter_unusual_predictions) if filter_unusual_predictions else ""
        options += " --filter-deletion-predictions %s" % str(filter_deletion_predictions) if filter_deletion_predictions else ""
        options += " --filter-insertion-predictions %s" % str(filter_insertion_predictions) if filter_insertion_predictions else ""
        options += " --heal-snps %s" % str(heal_snps) if heal_snps else ""

        self.execute(options)

        """
        General options:

          -n,--num-threads                      INT         number of worker threads [2]
          -N,--num-variants-per-thread          INT         worker thread batch size [500]
             --parameters-file                  FILE        json file with algorithm control parameters [optional]

        Inputs:
          -r,--reference                        FILE        reference fasta file [required]
          -b,--input-bam                        FILE        bam file with mapped reads [required]
          -g,--sample-name                      STRING      sample for which variants are called (In case of input BAM files with multiple samples) [optional if there is only one sample]
             --force-sample-name                STRING      force all read groups to have this sample name [off]
          -t,--target-file                      FILE        only process targets in this bed file [optional]
             --trim-ampliseq-primers            on/off      match reads to targets and trim the ends that reach outside them [off]
          -D,--downsample-to-coverage           INT         ?? [2000]
             --model-file                       FILE        HP recalibration model input file.
             --recal-model-hp-thres             INT         Lower threshold for HP recalibration.

        Outputs:
          -O,--output-dir                       DIRECTORY   base directory for all output files [current dir]
          -o,--output-vcf                       FILE        vcf file with variant calling results [required]
             --suppress-reference-genotypes     on/off      write reference calls into the filtered variants vcf [on]
             --suppress-no-calls                on/off      write filtered variants into the filtered variants vcf [on]
             --suppress-nocall-genotypes        on/off      do not report a genotype for filtered variants [on]

        Variant candidate generation (FreeBayes):
             --allow-snps                       on/off      allow generation of snp candidates [on]
             --allow-indels                     on/off      allow generation of indel candidates [on]
             --allow-mnps                       on/off      allow generation of mnp candidates [on]
             --allow-complex                    on/off      allow generation of block substitution candidates [off]
             --max-complex-gap                  INT         maximum number of reference alleles between two alternate alleles to allow merging of the alternate alleles [1]
          -m,--use-best-n-alleles               INT         maximum number of snp alleles [2]
          -M,--min-mapping-qv                   INT         do not use reads with mapping quality below this [4]
          -U,--read-snp-limit                   INT         do not use reads with number of snps above this [10]
          -z,--read-max-mismatch-fraction       FLOAT       do not use reads with fraction of mismatches above this [1.0]
             --gen-min-alt-allele-freq          FLOAT       minimum required alt allele frequency to generate a candidate [0.2]
             --gen-min-indel-alt-allele-freq    FLOAT       minimum required alt allele frequency to generate a homopolymer indel candidate [0.2]
             --gen-min-coverage                 INT         minimum required coverage to generate a candidate [6]

        External variant candidates:
          -c,--input-vcf                        FILE        vcf.gz file (+.tbi) with additional candidate variant locations and alleles [optional]
             --process-input-positions-only     on/off      only generate candidates at locations from input-vcf [off]
             --use-input-allele-only            on/off      only consider provided alleles for locations in input-vcf [off]

        Variant candidate scoring options:
             --min-delta-for-flow               FLOAT       minimum prediction delta for scoring flows [0.1]
             --max-flows-to-test                INT         maximum number of scoring flows [10]
             --outlier-probability              FLOAT       probability for outlier reads [0.01]
             --heavy-tailed                     INT         degrees of freedom in t-dist modeling signal residual heavy tail [3]
             --suppress-recalibration           on/off      Suppress homopolymer recalibration [on].
             --do-snp-realignment               on/off      Realign reads in the vicinity of candidate snp variants [on].
             --do-mnp-realignment               on/off      Realign reads in the vicinity of candidate mnp variants [do-snp-realignment].

        Advanced variant candidate scoring options:
             --use-sse-basecaller               on/off      Switch to use the vectorized version of the basecaller [on].
             --resolve-clipped-bases            on/off      If 'true', the basecaller is used to solve soft clipped bases [off].
             --prediction-precision             FLOAT       prior weight in bias estimator [30.0]
             --shift-likelihood-penalty         FLOAT       penalize log-likelihood for solutions involving large systematic bias [0.3]
             --minimum-sigma-prior              FLOAT       prior variance per data point, constant [0.085]
             --slope-sigma-prior                FLOAT       prior rate of increase of variance over minimum by signal [0.0084]
             --sigma-prior-weight               FLOAT       weight of prior estimate of variance compared to observations [1.0]
             --k-zero                           FLOAT       variance increase for adding systematic bias [3.0]
             --sse-relative-safety-level        FLOAT       dampen strand bias detection for SSE events for low coverage [0.025]
             --tune-sbias                       FLOAT       dampen strand bias detection for low coverage [0.01]

        Variant filtering:
          -k,--snp-min-coverage                 INT         filter out snps with total coverage below this [6]
          -C,--snp-min-cov-each-strand          INT         filter out snps with coverage on either strand below this [1]
          -B,--snp-min-variant-score            FLOAT       filter out snps with QUAL score below this [2.5]
          -s,--snp-strand-bias                  FLOAT       filter out snps with strand bias above this [0.95] given pval < snp-strand-bias-pval
             --snp-strand-bias-pval             FLOAT       filter out snps with pval below this [1.0] given strand bias > snp-strand-bias
          -A,--snp-min-allele-freq              FLOAT       minimum required alt allele frequency for non-reference snp calls [0.2]
             --mnp-min-coverage                 INT         filter out mnps with total coverage below this [snp-min-coverage]
             --mnp-min-cov-each-strand          INT         filter out mnps with coverage on either strand below this, [snp-min-coverage-each-strand]
             --mnp-min-variant-score            FLOAT       filter out mnps with QUAL score below this [snp-min-variant-score]
             --mnp-strand-bias                  FLOAT       filter out mnps with strand bias above this [snp-strand-bias] given pval < mnp-strand-bias-pval
             --mnp-strand-bias-pval             FLOAT       filter out mnps with pval below this [snp-strand-bias-pval] given strand bias > mnp-strand-bias
             --mnp-min-allele-freq              FLOAT       minimum required alt allele frequency for non-reference mnp calls [snp-min-allele-freq]
             --indel-min-coverage               INT         filter out indels with total coverage below this [30]
             --indel-min-cov-each-strand        INT         filter out indels with coverage on either strand below this [1]
             --indel-min-variant-score          FLOAT       filter out indels with QUAL score below this [2.5]
          -S,--indel-strand-bias                FLOAT       filter out indels with strand bias above this [0.95] given pval < indel-strand-bias-pval
             --indel-strand-bias-pval           FLOAT       filter out indels with pval below this [1.0] given strand bias > indel-strand-bias
             --indel-min-allele-freq            FLOAT       minimum required alt allele frequency for non-reference indel call [0.2]
             --hotspot-min-coverage             INT         filter out hotspot variants with total coverage below this [6]
             --hotspot-min-cov-each-strand      INT         filter out hotspot variants with coverage on either strand below this [1]
             --hotspot-min-variant-score        FLOAT       filter out hotspot variants with QUAL score below this [2.5]
             --hotspot-strand-bias              FLOAT       filter out hotspot variants with strand bias above this [0.95] given pval < hotspot-strand-bias-pval
             --hotspot-strand-bias-pval         FLOAT       filter out hotspot variants with pval below this [1.0] given strand bias > hotspot-strand-bias
          -H,--hotspot-min-allele-freq          FLOAT       minimum required alt allele frequency for non-reference hotspot variant call [0.2]
          -L,--hp-max-length                    INT         filter out indels in homopolymers above this [8]
          -e,--error-motifs                     FILE        table of systematic error motifs and their error rates [optional]
             --sse-prob-threshold               FLOAT       filter out variants in motifs with error rates above this [0.2]
             --min-ratio-reads-non-sse-strand   FLOAT       minimum required alt allele frequency for variants with error motifs on opposite strand [0.2]
             --data-quality-stringency          FLOAT       minimum mean log-likelihood delta per read [4.0]
             --read-rejection-threshold         FLOAT       filter variants where large numbers of reads are rejected as outliers [0.5]
             --filter-unusual-predictions       FLOAT       posterior log likelihood threshold for accepting bias estimate [0.3]
             --filter-deletion-predictions      FLOAT       check post-evaluation systematic bias in deletions [100.0]
             --filter-insertion-predictions     FLOAT       check post-evaluation systematic bias in insertions [100.0]
             --heal-snps
        """


if __name__ == "__main__":

    input_bam = "/media/mahajrod/d9e6e5ee-1bf7-4dba-934e-3f898d9611c8/Data/csa/alignment/001/Alsu24mc/tmap/uncorrected/001_trimmed_sorted_rm_pcr_chrom.bam"
    reference_fasta = "/media/mahajrod/d9e6e5ee-1bf7-4dba-934e-3f898d9611c8/Data/Alsu/reference/Alsu24mc/Alsu24mc.fasta"
    out_dir = "/media/mahajrod/d9e6e5ee-1bf7-4dba-934e-3f898d9611c8/Data/csa/alignment/001/Alsu24mc/tmap/uncorrected/tvc"
    out_vcf = "out.vcf"
    TVC = TVC()
    #TVC.variant_call(input_bam, reference_fasta, out_vcf, out_dir=out_dir)

    TVC.variant_caller_pipeline(input_bam, reference_fasta, tvc_root_dir="/home/mahajrod/Repositories/genetic/NGS_tools/tmap/tvc-4.2.3-build", )



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

from RouToolPa.Tools.Scaffolders.LRNAScaffolder import LRNAScaffolder
from RouToolPa.Tools.Scaffolders.SSPACE import SSPACE


SSPACE = SSPACE()
LRNAScaffolder = LRNAScaffolder()



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from RouToolPa.Tools.Abstract import Tool


class FaCut(Tool):
    def __init__(self, path="", max_threads=1):
        Tool.__init__(self, "facut", path=path, max_threads=max_threads)

    def filter_by_mean_quality(self, quality_threshold, output_prefix, forward_reads, reverse_reads=None,
                               quality_type="phred64", stat_file=None, name_type=None):

        options = " -q %s" % quality_type
        options += " -f %s" % forward_reads
        options += " -r %s" % reverse_reads if reverse_reads else ""
        options += " -u" if not reverse_reads else ""
        options += " -t %i" % quality_threshold
        options += " -p %s" % output_prefix
        options += " -n %s" % name_type if name_type else ""
        options += " > %s" % stat_file if stat_file else ""

        self.execute(options, cmd="filter_by_mean_quality")

    def split_fastq(self, input_file, output_prefix):

        options = " -i %s" % input_file
        options += " -p %s" % output_prefix

        self.cmd(options, cmd="split_fastq")

if __name__ == "__main__":
    pass


